# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

import ddt

from manilaclient.tests.functional import base


@ddt.ddt
class ManilaClientTestCommonReadOnly(base.BaseTestCase):

    @ddt.data('admin', 'user')
    def test_manila_version(self, role):
        self.clients[role].manila('', flags='--version')

    @ddt.data('admin', 'user')
    def test_help(self, role):
        help_text = self.clients[role].manila('help')
        lines = help_text.split('\n')
        self.assertFirstLineStartsWith(lines, 'usage: manila')

        commands = []
        cmds_start = lines.index('Positional arguments:')
        cmds_end = lines.index('Optional arguments:')
        command_pattern = re.compile(r'^ {4}([a-z0-9\-\_]+)')
        for line in lines[cmds_start:cmds_end]:
            match = command_pattern.match(line)
            if match:
                commands.append(match.group(1))
        commands = set(commands)
        wanted_commands = set((
            'absolute-limits', 'list', 'help', 'quota-show', 'access-list',
            'snapshot-list', 'access-allow', 'access-deny',
            'share-network-list', 'security-service-list'))
        self.assertFalse(wanted_commands - commands)

    @ddt.data('admin', 'user')
    def test_credentials(self, role):
        self.clients[role].manila('credentials')

    @ddt.data('admin', 'user')
    def test_list_extensions(self, role):
        roles = self.parser.listing(
            self.clients[role].manila('list-extensions'))
        self.assertTableStruct(roles, ['Name', 'Summary', 'Alias', 'Updated'])

    @ddt.data('admin', 'user')
    def test_endpoints(self, role):
        self.clients[role].manila('endpoints')



# Copyright 2014 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Common constants that can be used all over the manilaclient."""

# These are used for providing desired sorting params with list requests
SORT_DIR_VALUES = ('asc', 'desc')

SHARE_SORT_KEY_VALUES = (
    'id', 'status', 'size', 'host', 'share_proto',
    'availability_zone_id', 'availability_zone',
    'user_id', 'project_id',
    'created_at', 'updated_at',
    'display_name', 'name',
    'share_type_id', 'share_type',
    'share_network_id', 'share_network',
    'snapshot_id', 'snapshot',
)

SNAPSHOT_SORT_KEY_VALUES = (
    'id',
    'status',
    'size',
    'share_id',
    'user_id',
    'project_id',
    'progress',
    'name',
    'display_name',
)

SHARE_GROUP_SORT_KEY_VALUES = (
    'id',
    'name',
    'status',
    'host',
    'user_id',
    'project_id',
    'created_at',
    'availability_zone',
    'share_network',
    'share_network_id',
    'share_group_type',
    'share_group_type_id',
    'source_share_group_snapshot_id',
)

SHARE_GROUP_SNAPSHOT_SORT_KEY_VALUES = (
    'id',
    'name',
    'status',
    'host',
    'user_id',
    'project_id',
    'created_at',
    'share_group_id',
)

TASK_STATE_MIGRATION_SUCCESS = 'migration_success'
TASK_STATE_MIGRATION_ERROR = 'migration_error'
TASK_STATE_MIGRATION_CANCELLED = 'migration_cancelled'
TASK_STATE_MIGRATION_DRIVER_PHASE1_DONE = 'migration_driver_phase1_done'
TASK_STATE_DATA_COPYING_COMPLETED = 'data_copying_completed'

EXPERIMENTAL_HTTP_HEADER = 'X-OpenStack-Manila-API-Experimental'
V1_SERVICE_TYPE = 'share'
V2_SERVICE_TYPE = 'sharev2'
# Service type authority recommends using 'shared-file-system' as
# the service type. See: https://opendev.org/openstack/service-types-authority
SFS_SERVICE_TYPE = 'shared-file-system'

SERVICE_TYPES = {'1': V1_SERVICE_TYPE, '2': V2_SERVICE_TYPE}

EXTENSION_PLUGIN_NAMESPACE = 'manilaclient.common.apiclient.auth'
MESSAGE_SORT_KEY_VALUES = (
    'id', 'project_id', 'request_id', 'resource_type', 'action_id',
    'detail_id', 'resource_id', 'message_level', 'expires_at',
    'request_id', 'created_at'
)

STATUS_AVAILABLE = 'available'
STATUS_ERROR = 'error'
STATUS_ACTIVE = 'active'
STATUS_MANAGE_ERROR = 'manage_error'
STATUS_UNMANAGE_ERROR = 'unmanage_error'
STATUS_DELETING = 'deleting'
STATUS_CREATING = 'creating'
STATUS_SERVER_MIGRATING = 'server_migrating'

SNAPSHOT_SUPPORT = 'snapshot_support'
CREATE_SHARE_FROM_SNAPSHOT_SUPPORT = 'create_share_from_snapshot_support'
REVERT_TO_SNAPSHOT_SUPPORT = 'revert_to_snapshot_support'
MOUNT_SNAPSHOT_SUPPORT = 'mount_snapshot_support'

BOOL_SPECS = (
    SNAPSHOT_SUPPORT,
    CREATE_SHARE_FROM_SNAPSHOT_SUPPORT,
    REVERT_TO_SNAPSHOT_SUPPORT,
    MOUNT_SNAPSHOT_SUPPORT
)

REPLICA_GRADUATION_VERSION = '2.56'
REPLICA_PRE_GRADUATION_VERSION = '2.55'



#   Copyright 2021 Red Hat Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from osc_lib import utils as osc_lib_utils

from manilaclient.osc.v2 import (share_snapshot_instance_export_locations as
                                 osc_snapshot_instance_locations)
from manilaclient.tests.unit.osc import osc_utils
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes

COLUMNS = ['ID', 'Path', 'Is Admin only']


class TestShareSnapshotInstanceExportLocation(manila_fakes.TestShare):

    def setUp(self):
        super(TestShareSnapshotInstanceExportLocation, self).setUp()

        self.share_snapshot_instances_mock = (
            self.app.client_manager.share.share_snapshot_instances)
        self.share_snapshot_instances_mock.reset_mock()

        self.share_snapshot_instances_el_mock = (
            self.app.client_manager
                .share.share_snapshot_instance_export_locations)
        self.share_snapshot_instances_el_mock.reset_mock()


class TestShareSnapshotInstanceExportLocationList(
        TestShareSnapshotInstanceExportLocation):

    def setUp(self):
        super(TestShareSnapshotInstanceExportLocationList, self).setUp()

        self.share_snapshot_instance = (
            manila_fakes.FakeShareSnapshotIntances
            .create_one_snapshot_instance())

        self.share_snapshot_instances_export_locations = (
            manila_fakes.FakeShareSnapshotInstancesExportLocations
            .create_share_snapshot_instances(count=2)
        )

        self.share_snapshot_instances_mock.get.return_value = (
            self.share_snapshot_instance)

        self.share_snapshot_instances_el_mock.list.return_value = (
            self.share_snapshot_instances_export_locations)

        self.cmd = (osc_snapshot_instance_locations
                    .ShareSnapshotInstanceExportLocationList(self.app,
                                                             None))

    def test_share_snapshot_instance_export_location_list_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_snapshot_instance_export_location_list(self):
        values = (osc_lib_utils.get_dict_properties(
                  s._info, COLUMNS) for s in
                  self.share_snapshot_instances_export_locations)
        arglist = [
            self.share_snapshot_instance.id
        ]
        verifylist = [
            ('instance', self.share_snapshot_instance.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(values), list(data))


class TestShareSnapshotInstanceExportLocationShow(
        TestShareSnapshotInstanceExportLocation):

    def setUp(self):
        super(TestShareSnapshotInstanceExportLocationShow, self).setUp()

        self.share_snapshot_instance = (
            manila_fakes.FakeShareSnapshotIntances
            .create_one_snapshot_instance())

        self.share_snapshot_instances_export_location = (
            manila_fakes.FakeShareSnapshotInstancesExportLocations
            .create_one_snapshot_instance()
        )

        self.share_snapshot_instances_mock.get.return_value = (
            self.share_snapshot_instance)

        self.share_snapshot_instances_el_mock.get.return_value = (
            self.share_snapshot_instances_export_location)

        self.cmd = (osc_snapshot_instance_locations
                    .ShareSnapshotInstanceExportLocationShow(self.app,
                                                             None))

        self.data = (self.share_snapshot_instances_export_location.
                     _info.values())
        self.columns = (self.share_snapshot_instances_export_location.
                        _info.keys())

    def test_share_snapshot_instance_export_location_show_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_snapshot_instance_export_location_show(self):
        arglist = [
            self.share_snapshot_instance.id,
            self.share_snapshot_instances_export_location.id
        ]
        verifylist = [
            ('snapshot_instance', self.share_snapshot_instance.id),
            ('export_location',
             self.share_snapshot_instances_export_location.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.share_snapshot_instances_mock.get.assert_called_with(
            self.share_snapshot_instance.id)
        self.share_snapshot_instances_el_mock.get.assert_called_with(
            self.share_snapshot_instances_export_location.id,
            snapshot_instance=self.share_snapshot_instance)
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)



# Copyright 2014 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Share type access interface."""

from manilaclient import api_versions
from manilaclient import base
from manilaclient.common.apiclient import base as common_base


class ShareTypeAccess(common_base.Resource):
    def __repr__(self):
        return "<ShareTypeAccess: %s>" % self.id


class ShareTypeAccessManager(base.ManagerWithFind):
    """Manage :class:`ShareTypeAccess` resources."""

    resource_class = ShareTypeAccess

    def _do_list(self, share_type, action_name="share_type_access"):
        if share_type.is_public:
            return None

        return self._list(
            "/types/%(st_id)s/%(action_name)s" % {
                "st_id": common_base.getid(share_type),
                "action_name": action_name},
            "share_type_access")

    @api_versions.wraps("1.0", "2.6")
    def list(self, share_type, search_opts=None):
        return self._do_list(share_type, "os-share-type-access")

    @api_versions.wraps("2.7")  # noqa
    def list(self, share_type, search_opts=None):  # noqa
        return self._do_list(share_type, "share_type_access")

    def add_project_access(self, share_type, project):
        """Add a project to the given share type access list."""
        info = {'project': project}
        self._action('addProjectAccess', share_type, info)

    def remove_project_access(self, share_type, project):
        """Remove a project from the given share type access list."""
        info = {'project': project}
        self._action('removeProjectAccess', share_type, info)

    def _action(self, action, share_type, info, **kwargs):
        """Perform a share type action."""
        body = {action: info}
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = '/types/%s/action' % common_base.getid(share_type)
        return self.api.client.post(url, body=body)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient import api_versions
from manilaclient import base
from manilaclient.common.apiclient import base as common_base
from manilaclient.common import constants


class ShareReplicaExportLocation(common_base.Resource):
    """Resource class for a share replica export location."""

    def __repr__(self):
        return "<ShareReplicaExportLocation: %s>" % self.id

    def __getitem__(self, key):
        return self._info[key]


class ShareReplicaExportLocationManager(base.ManagerWithFind):
    """Manage :class:`ShareInstanceExportLocation` resources."""
    resource_class = ShareReplicaExportLocation

    @api_versions.wraps("2.47", constants.REPLICA_PRE_GRADUATION_VERSION)
    @api_versions.experimental_api
    def list(self, share_replica, search_opts=None):
        """List all share replica export locations."""
        share_replica_id = common_base.getid(share_replica)
        return self._list(
            "/share-replicas/%s/export-locations" % share_replica_id,
            "export_locations")

    @api_versions.wraps(constants.REPLICA_GRADUATION_VERSION)  # noqa
    def list(self, share_replica, search_opts=None):  # noqa F811
        """List all share replica export locations."""
        share_replica_id = common_base.getid(share_replica)
        return self._list(
            "/share-replicas/%s/export-locations" % share_replica_id,
            "export_locations")

    @api_versions.wraps("2.47", constants.REPLICA_PRE_GRADUATION_VERSION)
    @api_versions.experimental_api
    def get(self, share_replica, export_location):
        return self._get_replica_export_location(
            share_replica, export_location)

    @api_versions.wraps(constants.REPLICA_GRADUATION_VERSION)  # noqa
    def get(self, share_replica, export_location):  # noqa F811
        return self._get_replica_export_location(
            share_replica, export_location)

    def _get_replica_export_location(self, share_replica, export_location):
        """Get a share replica export location."""
        share_replica_id = common_base.getid(share_replica)
        export_location_id = common_base.getid(export_location)
        return self._get(
            ("/share-replicas/%(share_replica_id)s/export-locations/"
             "%(export_location_id)s") % {
                 "share_replica_id": share_replica_id,
                 "export_location_id": export_location_id,
            },
            "export_location")



# Copyright 2015 Mirantis inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient import api_versions
from manilaclient import base
from manilaclient.common.apiclient import base as common_base


class ShareInstanceExportLocation(common_base.Resource):
    """Resource class for a share export location."""

    def __repr__(self):
        return "<ShareInstanceExportLocation: %s>" % self.id

    def __getitem__(self, key):
        return self._info[key]


class ShareInstanceExportLocationManager(base.ManagerWithFind):
    """Manage :class:`ShareInstanceExportLocation` resources."""
    resource_class = ShareInstanceExportLocation

    @api_versions.wraps("2.9")
    def list(self, share_instance, search_opts=None):
        """List all share export locations."""
        share_instance_id = common_base.getid(share_instance)
        return self._list(
            "/share_instances/%s/export_locations" % share_instance_id,
            "export_locations")

    @api_versions.wraps("2.9")
    def get(self, share_instance, export_location):
        """Get a share export location."""
        share_instance_id = common_base.getid(share_instance)
        export_location_id = common_base.getid(export_location)
        return self._get(
            ("/share_instances/%(share_instance_id)s/export_locations/"
             "%(export_location_id)s") % {
                 "share_instance_id": share_instance_id,
                 "export_location_id": export_location_id,
            },
            "export_location")



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from osc_lib import utils as oscutils

from manilaclient.osc.v2 import availability_zones as osc_availability_zones
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes


class TestAvailabilityZones(manila_fakes.TestShare):

    def setUp(self):
        super(TestAvailabilityZones, self).setUp()

        self.zones_mock = self.app.client_manager.share.availability_zones
        self.zones_mock.reset_mock()


class TestShareAvailabilityZoneList(TestAvailabilityZones):

    availability_zones = manila_fakes.FakeShareAvailabilityZones.\
        create_share_availability_zones()
    COLUMNS = ("Id", "Name", "Created At", "Updated At")

    def setUp(self):
        super(TestShareAvailabilityZoneList, self).setUp()

        self.zones_mock.list.return_value = self.availability_zones

        # Get the command object to test
        self.cmd = osc_availability_zones.ShareAvailabilityZoneList(
            self.app, None)

        self.values = (oscutils.get_dict_properties(
            s._info, self.COLUMNS) for s in self.availability_zones)

    def test_share_list_availability_zone(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(self.COLUMNS, columns)
        self.assertCountEqual(list(self.values), list(data))



#   Copyright 2019 Red Hat Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#


from unittest import mock
import uuid

from osc_lib import exceptions
from osc_lib import utils as oscutils

from manilaclient import api_versions
from manilaclient.common import cliutils
from manilaclient.osc.v2 import share_snapshots as osc_share_snapshots
from manilaclient.tests.unit.osc import osc_utils
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes

COLUMNS = ['ID', 'Name']

COLUMNS_DETAIL = [
    'ID',
    'Name',
    'Status',
    'Description',
    'Created At',
    'Size',
    'Share ID',
    'Share Proto',
    'Share Size',
    'User ID'
]


class TestShareSnapshot(manila_fakes.TestShare):

    def setUp(self):
        super(TestShareSnapshot, self).setUp()

        self.shares_mock = self.app.client_manager.share.shares
        self.shares_mock.reset_mock()

        self.snapshots_mock = self.app.client_manager.share.share_snapshots
        self.snapshots_mock.reset_mock()
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            api_versions.MAX_VERSION)

        self.export_locations_mock = (
            self.app.client_manager.share.share_snapshot_export_locations)
        self.export_locations_mock.reset_mock()


class TestShareSnapshotCreate(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotCreate, self).setUp()

        self.share = manila_fakes.FakeShare.create_one_share()
        self.shares_mock.create.return_value = self.share

        self.shares_mock.get.return_value = self.share

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())
        self.snapshots_mock.create.return_value = self.share_snapshot

        self.cmd = osc_share_snapshots.CreateShareSnapshot(self.app, None)

        self.data = tuple(self.share_snapshot._info.values())
        self.columns = tuple(self.share_snapshot._info.keys())

    def test_share_snapshot_create_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_snapshot_create_required_args(self):
        arglist = [
            self.share.id
        ]
        verifylist = [
            ('share', self.share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.create.assert_called_with(
            share=self.share,
            force=False,
            name=None,
            description=None
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_share_snapshot_create_force(self):
        arglist = [
            self.share.id,
            '--force'
        ]
        verifylist = [
            ('share', self.share.id),
            ('force', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.create.assert_called_with(
            share=self.share,
            force=True,
            name=None,
            description=None
        )

        self.assertCountEqual(columns, columns)
        self.assertCountEqual(self.data, data)

    def test_share_snapshot_create(self):
        arglist = [
            self.share.id,
            '--name', self.share_snapshot.name,
            '--description', self.share_snapshot.description
        ]
        verifylist = [
            ('share', self.share.id),
            ('name', self.share_snapshot.name),
            ('description', self.share_snapshot.description)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.create.assert_called_with(
            share=self.share,
            force=False,
            name=self.share_snapshot.name,
            description=self.share_snapshot.description
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


class TestShareSnapshotDelete(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotDelete, self).setUp()

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.snapshots_mock.get.return_value = self.share_snapshot

        self.cmd = osc_share_snapshots.DeleteShareSnapshot(self.app, None)

    def test_share_snapshot_delete_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_snapshot_delete(self):
        arglist = [
            self.share_snapshot.id
        ]
        verifylist = [
            ('snapshot', [self.share_snapshot.id])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.snapshots_mock.delete.assert_called_with(self.share_snapshot)
        self.assertIsNone(result)

    def test_share_snapshot_delete_force(self):
        arglist = [
            self.share_snapshot.id,
            '--force'
        ]
        verifylist = [
            ('snapshot', [self.share_snapshot.id]),
            ('force', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.snapshots_mock.force_delete.assert_called_with(
            self.share_snapshot)
        self.assertIsNone(result)

    def test_share_snapshot_delete_multiple(self):
        share_snapshots = (
            manila_fakes.FakeShareSnapshot.create_share_snapshots(
                count=2))
        arglist = [
            share_snapshots[0].id,
            share_snapshots[1].id
        ]
        verifylist = [
            ('snapshot', [share_snapshots[0].id, share_snapshots[1].id])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.assertEqual(self.snapshots_mock.delete.call_count,
                         len(share_snapshots))
        self.assertIsNone(result)

    def test_share_snapshot_delete_exception(self):
        arglist = [
            self.share_snapshot.id
        ]
        verifylist = [
            ('snapshot', [self.share_snapshot.id])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.snapshots_mock.delete.side_effect = exceptions.CommandError()
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)


class TestShareSnapshotShow(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotShow, self).setUp()

        self.export_location = (
            manila_fakes.FakeShareExportLocation.create_one_export_location())

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot(
                attrs={
                    'export_locations': self.export_location
                }
            ))
        self.snapshots_mock.get.return_value = self.share_snapshot

        self.cmd = osc_share_snapshots.ShowShareSnapshot(self.app, None)

        self.data = self.share_snapshot._info.values()
        self.columns = self.share_snapshot._info.keys()

    def test_share_snapshot_show_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_snapshot_show(self):
        arglist = [
            self.share_snapshot.id
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cliutils.convert_dict_list_to_string = mock.Mock()
        cliutils.convert_dict_list_to_string.return_value = (
            self.export_location)

        columns, data = self.cmd.take_action(parsed_args)
        self.snapshots_mock.get.assert_called_with(self.share_snapshot.id)
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


class TestShareSnapshotSet(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotSet, self).setUp()

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.snapshots_mock.get.return_value = self.share_snapshot

        self.cmd = osc_share_snapshots.SetShareSnapshot(self.app, None)

    def test_set_snapshot_name(self):
        snapshot_name = 'snapshot-name-' + uuid.uuid4().hex
        arglist = [
            self.share_snapshot.id,
            '--name', snapshot_name
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('name', snapshot_name)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.snapshots_mock.update.assert_called_with(
            self.share_snapshot,
            display_name=parsed_args.name)
        self.assertIsNone(result)

    def test_set_snapshot_description(self):
        description = 'snapshot-description-' + uuid.uuid4().hex
        arglist = [
            self.share_snapshot.id,
            '--description', description
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('description', description)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.snapshots_mock.update.assert_called_with(
            self.share_snapshot,
            display_description=parsed_args.description)
        self.assertIsNone(result)

    def test_set_snapshot_status(self):
        arglist = [
            self.share_snapshot.id,
            '--status', 'available'
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('status', 'available')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.snapshots_mock.reset_state.assert_called_with(
            self.share_snapshot,
            parsed_args.status)
        self.assertIsNone(result)

    def test_set_snapshot_update_exception(self):
        snapshot_name = 'snapshot-name-' + uuid.uuid4().hex
        arglist = [
            self.share_snapshot.id,
            '--name', snapshot_name
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('name', snapshot_name)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.snapshots_mock.update.side_effect = Exception()

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_set_snapshot_status_exception(self):
        arglist = [
            self.share_snapshot.id,
            '--status', 'available'
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('status', 'available')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.snapshots_mock.reset_state.side_effect = Exception()

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestShareSnapshotUnset(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotUnset, self).setUp()

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.snapshots_mock.get.return_value = self.share_snapshot

        self.cmd = osc_share_snapshots.UnsetShareSnapshot(self.app, None)

    def test_unset_snapshot_name(self):
        arglist = [
            self.share_snapshot.id,
            '--name'
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('name', True)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.snapshots_mock.update.assert_called_with(
            self.share_snapshot,
            display_name=None)
        self.assertIsNone(result)

    def test_unset_snapshot_description(self):
        arglist = [
            self.share_snapshot.id,
            '--description'
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('description', True)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.snapshots_mock.update.assert_called_with(
            self.share_snapshot,
            display_description=None)
        self.assertIsNone(result)

    def test_unset_snapshot_name_exception(self):
        arglist = [
            self.share_snapshot.id,
            '--name'
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('name', True)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.snapshots_mock.update.side_effect = Exception()

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestShareSnapshotList(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotList, self).setUp()

        self.share_snapshots = (
            manila_fakes.FakeShareSnapshot.create_share_snapshots(
                count=2))
        self.snapshots_list = oscutils.sort_items(
            self.share_snapshots,
            'name:asc',
            str)

        self.snapshots_mock.list.return_value = self.snapshots_list

        self.values = (oscutils.get_dict_properties(
            s._info, COLUMNS) for s in self.snapshots_list)

        self.cmd = osc_share_snapshots.ListShareSnapshot(self.app, None)

    def test_list_snapshots(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.list.assert_called_with(
            search_opts={
                'offset': None,
                'limit': None,
                'all_tenants': False,
                'name': None,
                'status': None,
                'share_id': None,
                'usage': None,
                'name~': None,
                'description~': None,
                'description': None
            })

        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(self.values), list(data))

    def test_list_snapshots_all_projects(self):
        all_tenants_list = COLUMNS.copy()
        all_tenants_list.append('Project ID')
        list_values = (oscutils.get_dict_properties(
            s._info, all_tenants_list) for s in self.snapshots_list)

        arglist = [
            '--all-projects'
        ]

        verifylist = [
            ('all_projects', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.list.assert_called_with(
            search_opts={
                'offset': None,
                'limit': None,
                'all_tenants': True,
                'name': None,
                'status': None,
                'share_id': None,
                'usage': None,
                'name~': None,
                'description~': None,
                'description': None
            })

        self.assertEqual(all_tenants_list, columns)
        self.assertEqual(list(list_values), list(data))

    def test_list_snapshots_detail(self):
        values = (oscutils.get_dict_properties(
            s._info, COLUMNS_DETAIL) for s in self.snapshots_list)

        arglist = [
            '--detail'
        ]

        verifylist = [
            ('detail', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.list.assert_called_with(
            search_opts={
                'offset': None,
                'limit': None,
                'all_tenants': False,
                'name': None,
                'status': None,
                'share_id': None,
                'usage': None,
                'name~': None,
                'description~': None,
                'description': None
            })

        self.assertEqual(COLUMNS_DETAIL, columns)
        self.assertEqual(list(values), list(data))

    def test_list_snapshots_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.35")

        arglist = [
            '--description', 'Description'
        ]
        verifylist = [
            ('description', 'Description')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_list_snapshots_share_id(self):
        self.share = manila_fakes.FakeShare.create_one_share(
            attrs={'id': self.snapshots_list[0].id})

        self.shares_mock.get.return_value = self.share
        self.snapshots_mock.list.return_value = [self.snapshots_list[0]]

        values = (oscutils.get_dict_properties(
            s._info, COLUMNS) for s in [self.snapshots_list[0]])

        arglist = [
            '--share', self.share.id
        ]
        verifylist = [
            ('share', self.share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.list.assert_called_with(
            search_opts={
                'offset': None,
                'limit': None,
                'all_tenants': False,
                'name': None,
                'status': None,
                'share_id': self.share.id,
                'usage': None,
                'name~': None,
                'description~': None,
                'description': None
            })

        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(values), list(data))


class TestShareSnapshotAdopt(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotAdopt, self).setUp()

        self.share = manila_fakes.FakeShare.create_one_share()
        self.shares_mock.get.return_value = self.share

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.export_location = (
            manila_fakes.FakeShareExportLocation.create_one_export_location())

        self.snapshots_mock.manage.return_value = self.share_snapshot

        self.cmd = osc_share_snapshots.AdoptShareSnapshot(self.app, None)

        self.data = tuple(self.share_snapshot._info.values())
        self.columns = tuple(self.share_snapshot._info.keys())

    def test_share_snapshot_adopt_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_snapshot_adopt(self):
        arglist = [
            self.share.id,
            self.export_location.fake_path
        ]
        verifylist = [
            ('share', self.share.id),
            ('provider_location', self.export_location.fake_path)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.manage.assert_called_with(
            share=self.share,
            provider_location=self.export_location.fake_path,
            driver_options={},
            name=None,
            description=None
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_snapshot_adopt_name(self):
        name = 'name-' + uuid.uuid4().hex
        arglist = [
            self.share.id,
            self.export_location.fake_path,
            '--name', name,
        ]
        verifylist = [
            ('share', self.share.id),
            ('provider_location', self.export_location.fake_path),
            ('name', name)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.manage.assert_called_with(
            share=self.share,
            provider_location=self.export_location.fake_path,
            driver_options={},
            name=name,
            description=None
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_snapshot_adopt_driver_option(self):
        arglist = [
            self.share.id,
            self.export_location.fake_path,
            '--driver-option', 'key1=value1',
            '--driver-option', 'key2=value2'
        ]
        verifylist = [
            ('share', self.share.id),
            ('provider_location', self.export_location.fake_path),
            ('driver_option', {'key1': 'value1', 'key2': 'value2'})
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.manage.assert_called_with(
            share=self.share,
            provider_location=self.export_location.fake_path,
            driver_options={
                'key1': 'value1',
                'key2': 'value2'
            },
            name=None,
            description=None
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


class TestShareSnapshotAbandon(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotAbandon, self).setUp()

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.snapshots_mock.get.return_value = self.share_snapshot

        self.cmd = osc_share_snapshots.AbandonShareSnapshot(self.app, None)

    def test_share_snapshot_abandon_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_snapshot_abandon(self):
        arglist = [
            self.share_snapshot.id
        ]
        verifylist = [
            ('snapshot', [self.share_snapshot.id])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.snapshots_mock.unmanage.assert_called_with(self.share_snapshot)
        self.assertIsNone(result)

    def test_share_snapshot_abandon_multiple(self):
        share_snapshots = (
            manila_fakes.FakeShareSnapshot.create_share_snapshots(
                count=2))
        arglist = [
            share_snapshots[0].id,
            share_snapshots[1].id
        ]
        verifylist = [
            ('snapshot', [share_snapshots[0].id, share_snapshots[1].id])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.assertEqual(self.snapshots_mock.unmanage.call_count,
                         len(share_snapshots))
        self.assertIsNone(result)


class TestShareSnapshotAccessAllow(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotAccessAllow, self).setUp()

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.snapshots_mock.get.return_value = self.share_snapshot

        self.access_rule = (
            manila_fakes.FakeSnapshotAccessRule.create_one_access_rule())
        self.snapshots_mock.allow.return_value = self.access_rule._info

        self.cmd = osc_share_snapshots.ShareSnapshotAccessAllow(
            self.app, None)

    def test_share_snapshot_access_allow(self):
        arglist = [
            self.share_snapshot.id,
            'user',
            'demo'
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('access_type', 'user'),
            ('access_to', 'demo')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.allow.assert_called_with(
            snapshot=self.share_snapshot,
            access_type='user',
            access_to='demo'
        )
        self.assertEqual(tuple(self.access_rule._info.keys()), columns)
        self.assertCountEqual(self.access_rule._info.values(), data)

    def test_share_snapshot_access_allow_exception(self):
        arglist = [
            self.share_snapshot.id,
            'user',
            'demo'
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('access_type', 'user'),
            ('access_to', 'demo')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.snapshots_mock.allow.side_effect = exceptions.CommandError()
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)


class TestShareSnapshotAccessDeny(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotAccessDeny, self).setUp()

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.snapshots_mock.get.return_value = self.share_snapshot

        self.access_rule = (
            manila_fakes.FakeSnapshotAccessRule.create_one_access_rule())

        self.cmd = osc_share_snapshots.ShareSnapshotAccessDeny(
            self.app, None)

    def test_share_snapshot_access_deny(self):
        arglist = [
            self.share_snapshot.id,
            self.access_rule.id,
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('id', [self.access_rule.id]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.snapshots_mock.deny.assert_called_with(
            snapshot=self.share_snapshot,
            id=self.access_rule.id
        )
        self.assertIsNone(result)

    def test_share_snapshot_access_deny_multiple(self):
        access_rules = (
            manila_fakes.FakeSnapshotAccessRule.create_access_rules(
                count=2))

        arglist = [
            self.share_snapshot.id,
            access_rules[0].id,
            access_rules[1].id,
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('id', [access_rules[0].id, access_rules[1].id]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.assertEqual(self.snapshots_mock.deny.call_count,
                         len(access_rules))
        self.assertIsNone(result)

    def test_share_snapshot_access_deny_exception(self):
        arglist = [
            self.share_snapshot.id,
            self.access_rule.id,
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('id', [self.access_rule.id]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.snapshots_mock.deny.side_effect = exceptions.CommandError()
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)


class TestShareSnapshotAccessList(TestShareSnapshot):

    access_rules_columns = [
        'id',
        'access_type',
        'access_to',
        'state',
    ]

    def setUp(self):
        super(TestShareSnapshotAccessList, self).setUp()

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.snapshots_mock.get.return_value = self.share_snapshot

        self.access_rules = (
            manila_fakes.FakeSnapshotAccessRule.create_access_rules(
                count=2))

        self.snapshots_mock.access_list.return_value = self.access_rules
        self.cmd = osc_share_snapshots.ShareSnapshotAccessList(
            self.app, None)

        self.values = (oscutils.get_dict_properties(
            a._info, self.access_rules_columns) for a in self.access_rules)

    def test_share_snapshot_access_list(self):
        arglist = [
            self.share_snapshot.id
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.snapshots_mock.access_list.assert_called_with(
            self.share_snapshot)

        self.assertEqual(self.access_rules_columns, columns)
        self.assertCountEqual(self.values, data)


class TestShareSnapshotExportLocationList(TestShareSnapshot):

    columns = ["id", "path"]

    def setUp(self):
        super(TestShareSnapshotExportLocationList, self).setUp()

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.snapshots_mock.get.return_value = self.share_snapshot

        self.export_locations = (
            manila_fakes.FakeSnapshotExportLocation.create_export_locations()
        )

        self.export_locations_mock.list.return_value = self.export_locations
        self.values = (oscutils.get_dict_properties(
            e._info, self.columns) for e in self.export_locations)

        self.cmd = osc_share_snapshots.ShareSnapshotListExportLocation(
            self.app, None)

    def test_snapshot_export_locations_list(self):
        arglist = [
            self.share_snapshot.id
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.export_locations_mock.list.assert_called_with(
            snapshot=self.share_snapshot)

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.values, data)


class TestShareSnapshotExportLocationShow(TestShareSnapshot):

    def setUp(self):
        super(TestShareSnapshotExportLocationShow, self).setUp()

        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.snapshots_mock.get.return_value = self.share_snapshot

        self.export_location = (
            manila_fakes.FakeSnapshotExportLocation.create_one_export_location()  # noqa E501
        )

        self.export_locations_mock.get.return_value = self.export_location

        self.cmd = osc_share_snapshots.ShareSnapshotShowExportLocation(
            self.app, None)

    def test_snapshot_export_locations_list(self):
        arglist = [
            self.share_snapshot.id,
            self.export_location.id
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id),
            ('export_location', self.export_location.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.export_locations_mock.get.assert_called_with(
            export_location=self.export_location.id,
            snapshot=self.share_snapshot)

        self.assertEqual(tuple(self.export_location._info.keys()), columns)
        self.assertCountEqual(self.export_location._info.values(), data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
from osc_lib.command import command
from osc_lib import utils as osc_utils

from manilaclient.common._i18n import _


class ShareReplicaListExportLocation(command.Lister):
    """List export locations of a share replica."""

    _description = _("List export locations of a share replica.")

    def get_parser(self, prog_name):
        parser = super(
            ShareReplicaListExportLocation, self).get_parser(prog_name)
        parser.add_argument(
            "replica",
            metavar="<replica>",
            help=_("ID of the share replica.")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        columns = [
            'ID',
            'Availability Zone',
            'Replica State',
            'Preferred',
            'Path',
        ]

        replica = osc_utils.find_resource(
            share_client.share_replicas,
            parsed_args.replica)

        export_locations = share_client.share_replica_export_locations.list(
            replica)

        data = (osc_utils.get_dict_properties(
            location._info, columns) for location in export_locations)

        return (columns, data)


class ShareReplicaShowExportLocation(command.ShowOne):
    """Show details of a share replica's export location."""

    _description = _("Show details of a share replica's export location.")

    def get_parser(self, prog_name):
        parser = super(
            ShareReplicaShowExportLocation, self).get_parser(prog_name)
        parser.add_argument(
            "replica",
            metavar="<replica>",
            help=_("ID of the share replica.")
        )
        parser.add_argument(
            "export_location",
            metavar="<export-location>",
            help=_("ID of the share replica export location.")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        replica = osc_utils.find_resource(
            share_client.share_replicas,
            parsed_args.replica)

        export_location = share_client.share_replica_export_locations.get(
            replica, parsed_args.export_location)

        return self.dict2columns(export_location._info)



# Copyright 2016 Huawei inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from oslo_utils import uuidutils
import testtools

from manilaclient import config
from manilaclient.tests.functional import base
from manilaclient.tests.functional import utils

CONF = config.CONF


@ddt.ddt
@testtools.skipUnless(CONF.run_snapshot_tests,
                      'Snapshot tests disabled.')
@utils.skip_if_microversion_not_supported('2.19')
class SnapshotInstancesTest(base.BaseTestCase):

    def setUp(self):
        super(SnapshotInstancesTest, self).setUp()
        self.share = self.create_share(
            client=self.get_user_client())
        self.snapshot = self.create_snapshot(share=self.share['id'],
                                             client=self.get_user_client())

    def test_list_all_snapshot_instances(self):
        snapshot_instances = self.admin_client.list_snapshot_instances()

        self.assertGreater(len(snapshot_instances), 0)
        expected_keys = ('ID', 'Snapshot ID', 'Status')
        for si in snapshot_instances:
            for key in expected_keys:
                self.assertIn(key, si)
            self.assertTrue(uuidutils.is_uuid_like(si['ID']))
            self.assertTrue(uuidutils.is_uuid_like(si['Snapshot ID']))

    def test_list_all_snapshot_instances_details(self):
        snapshot_instances = self.admin_client.list_snapshot_instances(
            detailed=True)

        self.assertGreater(len(snapshot_instances), 0)
        expected_keys = ('ID', 'Snapshot ID', 'Status', 'Created_at',
                         'Updated_at', 'Share_id', 'Share_instance_id',
                         'Progress', 'Provider_location')
        for si in snapshot_instances:
            for key in expected_keys:
                self.assertIn(key, si)
            for key in ('ID', 'Snapshot ID', 'Share_id', 'Share_instance_id'):
                self.assertTrue(
                    uuidutils.is_uuid_like(si[key]))

    def test_list_snapshot_instance_with_snapshot(self):
        snapshot_instances = self.admin_client.list_snapshot_instances(
            snapshot_id=self.snapshot['id'])

        self.assertEqual(1, len(snapshot_instances))
        expected_keys = ('ID', 'Snapshot ID', 'Status')
        for si in snapshot_instances:
            for key in expected_keys:
                self.assertIn(key, si)
            self.assertTrue(uuidutils.is_uuid_like(si['ID']))
            self.assertTrue(uuidutils.is_uuid_like(si['Snapshot ID']))

    def test_list_snapshot_instance_with_columns(self):
        snapshot_instances = self.admin_client.list_snapshot_instances(
            self.snapshot['id'], columns='id,status')

        self.assertGreater(len(snapshot_instances), 0)
        expected_keys = ('Id', 'Status')
        unexpected_keys = ('Snapshot ID', )
        for si in snapshot_instances:
            for key in expected_keys:
                self.assertIn(key, si)
            for key in unexpected_keys:
                self.assertNotIn(key, si)
            self.assertTrue(uuidutils.is_uuid_like(si['Id']))

    def test_get_snapshot_instance(self):
        snapshot_instances = self.admin_client.list_snapshot_instances(
            self.snapshot['id'])

        snapshot_instance = self.admin_client.get_snapshot_instance(
            snapshot_instances[0]['ID'])
        self.assertGreater(len(snapshot_instance), 0)
        expected_keys = ('id', 'snapshot_id', 'status', 'created_at',
                         'updated_at', 'share_id', 'share_instance_id',
                         'progress', 'provider_location')

        for key in expected_keys:
            self.assertIn(key, snapshot_instance)
        for key in ('id', 'snapshot_id', 'share_id', 'share_instance_id'):
            self.assertTrue(
                uuidutils.is_uuid_like(snapshot_instance[key]))

    def test_snapshot_instance_reset_state(self):
        snapshot_instances = self.admin_client.list_snapshot_instances(
            self.snapshot['id'])
        self.admin_client.reset_snapshot_instance(
            snapshot_instances[0]['ID'], 'error')
        snapshot_instance = self.admin_client.get_snapshot_instance(
            snapshot_instances[0]['ID'])

        self.assertEqual('error', snapshot_instance['status'])
        self.admin_client.reset_snapshot_instance(snapshot_instance['id'],
                                                  'available')



# Copyright 2012 NetApp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Interface for shares extension."""

import collections
import ipaddress
from oslo_utils import uuidutils
import re
import string

from manilaclient import api_versions
from manilaclient import base
from manilaclient.common.apiclient import base as common_base
from manilaclient.common import constants
from manilaclient import exceptions
from manilaclient.v2 import share_instances


class Share(common_base.Resource):
    """A share is an extra block level storage to the OpenStack instances."""
    def __repr__(self):
        return "<Share: %s>" % self.id

    def update(self, **kwargs):
        """Update this share."""
        self.manager.update(self, **kwargs)

    def unmanage(self, **kwargs):
        """Unmanage this share."""
        self.manager.unmanage(self, **kwargs)

    def migration_start(self, host, force_host_assisted_migration,
                        preserve_metadata, writable, nondisruptive,
                        preserve_snapshots, new_share_network_id=None,
                        new_share_type_id=None):
        """Migrate the share to a new host."""
        self.manager.migration_start(self, host, force_host_assisted_migration,
                                     preserve_metadata, writable,
                                     nondisruptive, preserve_snapshots,
                                     new_share_network_id, new_share_type_id)

    def migration_complete(self):
        """Complete migration of a share."""
        self.manager.migration_complete(self)

    def migration_cancel(self):
        """Attempts to cancel migration of a share."""
        self.manager.migration_cancel(self)

    def migration_get_progress(self):
        """Obtain progress of migration of a share."""
        return self.manager.migration_get_progress(self)

    def reset_task_state(self, task_state):
        """Reset the task state of a given share."""
        self.manager.reset_task_state(self, task_state)

    def delete(self, share_group_id=None):
        """Delete this share."""
        self.manager.delete(self, share_group_id=share_group_id)

    def force_delete(self):
        """Delete the specified share ignoring its current state."""
        self.manager.force_delete(self)

    def allow(self, access_type, access, access_level, metadata=None):
        """Allow access to a share."""
        return self.manager.allow(
            self, access_type, access, access_level, metadata)

    def deny(self, id):
        """Deny access from IP to a share."""
        return self.manager.deny(self, id)

    def access_list(self):
        """Get access list from a share."""
        return self.manager.access_list(self)

    def update_all_metadata(self, metadata):
        """Update all metadata of this share."""
        return self.manager.update_all_metadata(self, metadata)

    def reset_state(self, state):
        """Update the share with the provided state."""
        self.manager.reset_state(self, state)

    def extend(self, new_size):
        """Extend the size of the specified share."""
        self.manager.extend(self, new_size)

    def shrink(self, new_size):
        """Shrink the size of the specified share."""
        self.manager.shrink(self, new_size)

    def list_instances(self):
        """List instances of the specified share."""
        self.manager.list_instances(self)

    def revert_to_snapshot(self, snapshot):
        """Reverts a share (in place) to a snapshot."""
        self.manager.revert_to_snapshot(self, snapshot)


class ShareManager(base.ManagerWithFind):
    """Manage :class:`Share` resources."""
    resource_class = Share

    def create(self, share_proto, size, snapshot_id=None, name=None,
               description=None, metadata=None, share_network=None,
               share_type=None, is_public=False, availability_zone=None,
               share_group_id=None):
        """Create a share.

        :param share_proto: text - share protocol for new share available
            values are NFS, CIFS, CephFS, GlusterFS, HDFS and MAPRFS.
        :param size: int - size in GiB
        :param snapshot_id: text - ID of the snapshot
        :param name: text - name of new share
        :param description: text - description of a share
        :param metadata: dict - optional metadata to set on share creation
        :param share_network: either instance of ShareNetwork or text with ID
        :param share_type: either instance of ShareType or text with ID
        :param is_public: bool, whether to set share as public or not.
        :param share_group_id: text - ID of the share group to which the share
            should belong
        :rtype: :class:`Share`
        """
        share_metadata = metadata if metadata is not None else dict()
        body = {
            'size': size,
            'snapshot_id': snapshot_id,
            'name': name,
            'description': description,
            'metadata': share_metadata,
            'share_proto': share_proto,
            'share_network_id': common_base.getid(share_network),
            'share_type': common_base.getid(share_type),
            'is_public': is_public,
            'availability_zone': availability_zone,
        }
        if share_group_id:
            body['share_group_id'] = share_group_id

        return self._create('/shares', {'share': body}, 'share')

    @api_versions.wraps("2.29")
    @api_versions.experimental_api
    def migration_start(self, share, host, force_host_assisted_migration,
                        preserve_metadata, writable, nondisruptive,
                        preserve_snapshots, new_share_network_id=None,
                        new_share_type_id=None):
        return self._action(
            "migration_start", share, {
                "host": host,
                "force_host_assisted_migration": force_host_assisted_migration,
                "preserve_metadata": preserve_metadata,
                "preserve_snapshots": preserve_snapshots,
                "writable": writable,
                "nondisruptive": nondisruptive,
                "new_share_network_id": new_share_network_id,
                "new_share_type_id": new_share_type_id,
            })

    @api_versions.wraps("2.22")
    @api_versions.experimental_api
    def reset_task_state(self, share, task_state):
        """Update the provided share with the provided task state.

        :param share: either share object or text with its ID.
        :param task_state: text with new task state to set for share.
        """
        return self._action('reset_task_state', share,
                            {"task_state": task_state})

    @api_versions.wraps("2.22")
    @api_versions.experimental_api
    def migration_complete(self, share):
        """Completes migration for a given share.

        :param share: The :class:'share' to complete migration
        """
        return self._action('migration_complete', share)

    @api_versions.wraps("2.22")
    @api_versions.experimental_api
    def migration_cancel(self, share):
        """Attempts to cancel migration for a given share.

        :param share: The :class:'share' to cancel migration
        """
        return self._action('migration_cancel', share)

    @api_versions.wraps("2.22")
    @api_versions.experimental_api
    def migration_get_progress(self, share):
        """Obtains progress of share migration for a given share.

        :param share: The :class:'share' to obtain migration progress
        """
        return self._action('migration_get_progress', share)

    def _do_manage(self, service_host, protocol, export_path,
                   driver_options=None, share_type=None,
                   name=None, description=None, is_public=None,
                   share_server_id=None, resource_path="/shares/manage"):
        """Manage some existing share.

        :param service_host: text - host where manila share service is running
        :param protocol: text - share protocol that is used
        :param export_path: text - export path of share
        :param driver_options: dict - custom set of key-values
        :param share_type: text - share type that should be used for share
        :param name: text - name of new share
        :param description: - description for new share
        :param is_public: - visibility for new share
        :param share_server_id: text - id of share server associated with share
        """
        driver_options = driver_options if driver_options else dict()
        body = {
            'service_host': service_host,
            'share_type': share_type,
            'protocol': protocol,
            'export_path': export_path,
            'driver_options': driver_options,
            'name': name,
            'description': description,
            'share_server_id': share_server_id,
        }

        if is_public is not None:
            body['is_public'] = is_public

        return self._create(resource_path, {'share': body}, 'share')

    @api_versions.wraps("1.0", "2.6")
    def manage(self, service_host, protocol, export_path, driver_options=None,
               share_type=None, name=None, description=None):
        return self._do_manage(
            service_host, protocol, export_path, driver_options=driver_options,
            share_type=share_type, name=name, description=description,
            resource_path="/os-share-manage")

    @api_versions.wraps("2.7", "2.7")  # noqa
    def manage(self, service_host, protocol, export_path, driver_options=None,  # noqa
               share_type=None, name=None, description=None):
        return self._do_manage(
            service_host, protocol, export_path, driver_options=driver_options,
            share_type=share_type, name=name, description=description,
            resource_path="/shares/manage")

    @api_versions.wraps("2.8", "2.48")  # noqa
    def manage(self, service_host, protocol, export_path, driver_options=None,  # noqa
               share_type=None, name=None, description=None, is_public=False):
        return self._do_manage(
            service_host, protocol, export_path, driver_options=driver_options,
            share_type=share_type, name=name, description=description,
            is_public=is_public, resource_path="/shares/manage")

    @api_versions.wraps("2.49")  # noqa
    def manage(self, service_host, protocol, export_path, driver_options=None,  # noqa
               share_type=None, name=None, description=None, is_public=False,
               share_server_id=None):
        return self._do_manage(
            service_host, protocol, export_path, driver_options=driver_options,
            share_type=share_type, name=name, description=description,
            is_public=is_public, share_server_id=share_server_id,
            resource_path="/shares/manage")

    @api_versions.wraps("1.0", "2.6")
    def unmanage(self, share):
        """Unmanage a share.

        :param share: either share object or text with its ID.
        """
        return self.api.client.post(
            "/os-share-unmanage/%s/unmanage" % common_base.getid(share))

    @api_versions.wraps("2.7")  # noqa
    def unmanage(self, share):   # noqa
        """Unmanage a share.

        :param share: either share object or text with its ID.
        """
        return self._action("unmanage", share)

    @api_versions.wraps("2.27")
    def revert_to_snapshot(self, share, snapshot):
        """Reverts a share (in place) to a snapshot.

        The snapshot must be the most recent one known to manila.
        :param share: either share object or text with its ID.
        :param snapshot: either snapshot object or text with its ID.
        """

        snapshot_id = common_base.getid(snapshot)
        info = {'snapshot_id': snapshot_id}
        return self._action('revert', share, info=info)

    def get(self, share):
        """Get a share.

        :param share: either share object or text with its ID.
        :rtype: :class:`Share`
        """
        share_id = common_base.getid(share)
        return self._get("/shares/%s" % share_id, "share")

    def update(self, share, **kwargs):
        """Updates a share.

        :param share: either share object or text with its ID.
        :rtype: :class:`Share`
        """
        if not kwargs:
            return

        body = {'share': kwargs, }
        share_id = common_base.getid(share)
        return self._update("/shares/%s" % share_id, body)

    @api_versions.wraps("1.0", "2.34")
    def list(self, detailed=True, search_opts=None,
             sort_key=None, sort_dir=None):
        """Get a list of all shares."""
        search_opts = search_opts or {}
        search_opts.pop("export_location", None)
        return self.do_list(detailed=detailed, search_opts=search_opts,
                            sort_key=sort_key, sort_dir=sort_dir)

    @api_versions.wraps("2.35")   # noqa
    def list(self, detailed=True, search_opts=None,   # noqa
             sort_key=None, sort_dir=None):
        """Get a list of all shares."""
        return self.do_list(detailed=detailed, search_opts=search_opts,
                            sort_key=sort_key, sort_dir=sort_dir)

    def do_list(self, detailed=True, search_opts=None,
                sort_key=None, sort_dir=None):
        """Get a list of all shares.

        :param detailed: Whether to return detailed share info or not.
        :param search_opts: dict with search options to filter out shares.
            available keys are below (('name1', 'name2', ...), 'type'):
            - ('all_tenants', int)
            - ('is_public', bool)
            - ('metadata', dict)
            - ('extra_specs', dict)
            - ('limit', int)
            - ('offset', int)
            - ('name', text)
            - ('status', text)
            - ('host', text)
            - ('share_server_id', text)
            - (('share_network_id', 'share_network'), text)
            - (('share_type_id', 'share_type'), text)
            - (('snapshot_id', 'snapshot'), text)
            Note, that member context will have restricted set of
            available search opts. For admin context filtering also available
            by each share attr from its Model. So, this list is not full for
            admin context.
        :param sort_key: Key to be sorted (i.e. 'created_at' or 'status').
        :param sort_dir: Sort direction, should be 'desc' or 'asc'.
        :rtype: list of :class:`Share`
        """
        if search_opts is None:
            search_opts = {}

        if sort_key is not None:
            if sort_key in constants.SHARE_SORT_KEY_VALUES:
                search_opts['sort_key'] = sort_key
                # NOTE(vponomaryov): Replace aliases with appropriate keys
                if sort_key == 'share_type':
                    search_opts['sort_key'] = 'share_type_id'
                elif sort_key == 'snapshot':
                    search_opts['sort_key'] = 'snapshot_id'
                elif sort_key == 'share_network':
                    search_opts['sort_key'] = 'share_network_id'
                elif sort_key == 'availability_zone':
                    search_opts['sort_key'] = 'availability_zone_id'
            else:
                raise ValueError('sort_key must be one of the following: %s.'
                                 % ', '.join(constants.SHARE_SORT_KEY_VALUES))

        if sort_dir is not None:
            if sort_dir in constants.SORT_DIR_VALUES:
                search_opts['sort_dir'] = sort_dir
            else:
                raise ValueError('sort_dir must be one of the following: %s.'
                                 % ', '.join(constants.SORT_DIR_VALUES))

        if 'is_public' not in search_opts:
            search_opts['is_public'] = True

        export_location = search_opts.pop('export_location', None)
        if export_location:
            if uuidutils.is_uuid_like(export_location):
                search_opts['export_location_id'] = export_location
            else:
                search_opts['export_location_path'] = export_location

        query_string = self._build_query_string(search_opts)

        if detailed:
            path = "/shares/detail%s" % (query_string,)
        else:
            path = "/shares%s" % (query_string,)

        return self._list(path, 'shares')

    def delete(self, share, share_group_id=None):
        """Delete a share.

        :param share: either share object or text with its ID.
        :param share_group_id: text - ID of the share group to which the share
            belongs
        """
        url = "/shares/%s" % common_base.getid(share)
        if share_group_id:
            url += "?share_group_id=%s" % share_group_id
        self._delete(url)

    def _do_force_delete(self, share, action_name):
        """Delete a share forcibly - share status will be avoided.

        :param share: either share object or text with its ID.
        """
        return self._action(action_name, share)

    @api_versions.wraps("1.0", "2.6")
    def force_delete(self, share):
        return self._do_force_delete(share, "os-force_delete")

    @api_versions.wraps("2.7")  # noqa
    def force_delete(self, share):   # noqa
        return self._do_force_delete(share, "force_delete")

    @staticmethod
    def _validate_common_name(access):
        if len(access) == 0 or len(access) > 64:
            exc_str = ('Invalid CN (common name). Must be 1-64 chars long.')
            raise exceptions.CommandError(exc_str)

    '''
    for the reference specification for AD usernames, reference below links:

    1:https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/
            windows-server-2008-R2-and-2008/cc733146(v=ws.11)
    2:https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/
            windows-server-2000/bb726984(v=technet.10)
    '''
    @staticmethod
    def _validate_username(access):
        sole_periods_spaces_re = r'[\s|\.]+$'
        valid_username_re = r'.[^\"\/\\\[\]\:\;\|\=\,\+\*\?\<\>]{3,254}$'
        username = access

        if re.match(sole_periods_spaces_re, username):
            exc_str = ('Invalid user or group name,cannot consist solely '
                       'of periods or spaces.')
            raise exceptions.CommandError(exc_str)

        if not re.match(valid_username_re, username):
            exc_str = ('Invalid user or group name. Must be 4-255 characters '
                       'and consist of alphanumeric characters and '
                       'exclude special characters "/\\[]:;|=,+*?<>')
            raise exceptions.CommandError(exc_str)

    @staticmethod
    def _validate_cephx_id(cephx_id):
        if not cephx_id:
            raise exceptions.CommandError(
                'Ceph IDs may not be empty.')

        # This restriction may be lifted in Ceph in the future:
        # http://tracker.ceph.com/issues/14626
        if not set(cephx_id) <= set(string.printable):
            raise exceptions.CommandError(
                'Ceph IDs must consist of ASCII printable characters.')

        # Periods are technically permitted, but we restrict them here
        # to avoid confusion where users are unsure whether they should
        # include the "client." prefix: otherwise they could accidentally
        # create "client.client.foobar".
        if '.' in cephx_id:
            raise exceptions.CommandError(
                'Ceph IDs may not contain periods.')

    def _validate_access(self, access_type, access, valid_access_types=None,
                         enable_ipv6=False):
        if not valid_access_types:
            valid_access_types = ('ip', 'user', 'cert')

        if access_type in valid_access_types:
            if access_type == 'ip':
                try:
                    if enable_ipv6:
                        ipaddress.ip_network(str(access))
                    else:
                        ipaddress.IPv4Network(str(access))
                except ValueError as error:
                    raise exceptions.CommandError(str(error))
            elif access_type == 'user':
                self._validate_username(access)
            elif access_type == 'cert':
                # 'access' is used as the certificate's CN (common name)
                # to which access is allowed or denied by the backend.
                # The standard allows for just about any string in the
                # common name. The meaning of a string depends on its
                # interpretation and is limited to 64 characters.
                self._validate_common_name(access.strip())
            elif access_type == 'cephx':
                self._validate_cephx_id(access.strip())
        else:
            msg = ('Only following access types are supported: %s' %
                   ', '.join(valid_access_types))
            raise exceptions.CommandError(msg)

    def _do_allow(self, share, access_type, access, access_level, action_name,
                  metadata=None):
        """Allow access to a share.

        :param share: either share object or text with its ID.
        :param access_type: string that represents access type ('ip','domain')
        :param access: string that represents access ('127.0.0.1')
        :param access_level: string that represents access level ('rw', 'ro')
        :param metadata: A dict of key/value pairs to be set
        """
        access_params = {
            'access_type': access_type,
            'access_to': access,
        }
        if access_level:
            access_params['access_level'] = access_level
        if metadata:
            access_params['metadata'] = metadata
        access = self._action(action_name, share,
                              access_params)[1]["access"]
        return access

    @api_versions.wraps("1.0", "2.6")
    def allow(self, share, access_type, access, access_level, metadata=None):
        self._validate_access(access_type, access)
        return self._do_allow(
            share, access_type, access, access_level, "os-allow_access")

    @api_versions.wraps("2.7", "2.12")  # noqa
    def allow(self, share, access_type, access, access_level, metadata=None):   # noqa
        self._validate_access(access_type, access)
        return self._do_allow(
            share, access_type, access, access_level, "allow_access")

    @api_versions.wraps("2.13", "2.37")  # noqa
    def allow(self, share, access_type, access, access_level, metadata=None):   # noqa
        valid_access_types = ('ip', 'user', 'cert', 'cephx')
        self._validate_access(access_type, access, valid_access_types)
        return self._do_allow(
            share, access_type, access, access_level, "allow_access")

    @api_versions.wraps("2.38", "2.44")  # noqa
    def allow(self, share, access_type, access, access_level, metadata=None):   # noqa
        valid_access_types = ('ip', 'user', 'cert', 'cephx')
        self._validate_access(access_type, access, valid_access_types,
                              enable_ipv6=True)
        return self._do_allow(
            share, access_type, access, access_level, "allow_access")

    @api_versions.wraps("2.45")  # noqa
    def allow(self, share, access_type, access, access_level, metadata=None):   # noqa
        valid_access_types = ('ip', 'user', 'cert', 'cephx')
        self._validate_access(access_type, access, valid_access_types,
                              enable_ipv6=True)
        return self._do_allow(
            share, access_type, access, access_level, "allow_access", metadata)

    def _do_deny(self, share, access_id, action_name):
        """Deny access to a share.

        :param share: either share object or text with its ID.
        :param access_id: ID of share access rule
        """
        return self._action(action_name, share, {"access_id": access_id})

    @api_versions.wraps("1.0", "2.6")
    def deny(self, share, access_id):
        return self._do_deny(share, access_id, "os-deny_access")

    @api_versions.wraps("2.7")  # noqa
    def deny(self, share, access_id):   # noqa
        return self._do_deny(share, access_id, "deny_access")

    def _do_access_list(self, share, action_name):
        """Get access list to a share.

        :param share: either share object or text with its ID.
        """
        access_list = self._action(action_name, share)[1]["access_list"]
        if access_list:
            t = collections.namedtuple('Access', list(access_list[0]))
            return [t(*value.values()) for value in access_list]
        else:
            return []

    @api_versions.wraps("1.0", "2.6")
    def access_list(self, share):
        return self._do_access_list(share, "os-access_list")

    @api_versions.wraps("2.7", "2.44")  # noqa
    def access_list(self, share):   # noqa
        return self._do_access_list(share, "access_list")

    def get_metadata(self, share):
        """Get metadata of a share.

        :param share: either share object or text with its ID.
        """
        return self._get("/shares/%s/metadata" % common_base.getid(share),
                         "metadata")

    def set_metadata(self, share, metadata):
        """Set or update metadata for share.

        :param share: either share object or text with its ID.
        :param metadata: A list of keys to be set.
        """
        body = {'metadata': metadata}
        return self._create("/shares/%s/metadata" % common_base.getid(share),
                            body, "metadata")

    def delete_metadata(self, share, keys):
        """Delete specified keys from shares metadata.

        :param share: either share object or text with its ID.
        :param keys: A list of keys to be removed.
        """
        share_id = common_base.getid(share)
        for key in keys:
            self._delete("/shares/%(share_id)s/metadata/%(key)s" % {
                'share_id': share_id, 'key': key})

    def update_all_metadata(self, share, metadata):
        """Update all metadata of a share.

        :param share: either share object or text with its ID.
        :param metadata: A list of keys to be updated.
        """
        body = {'metadata': metadata}
        return self._update("/shares/%s/metadata" % common_base.getid(share),
                            body)

    def _action(self, action, share, info=None, **kwargs):
        """Perform a share 'action'.

        :param action: text with action name.
        :param share: either share object or text with its ID.
        :param info: dict with data for specified 'action'.
        :param kwargs: dict with data to be provided for action hooks.
        """
        body = {action: info}
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = '/shares/%s/action' % common_base.getid(share)
        return self.api.client.post(url, body=body)

    def _do_reset_state(self, share, state, action_name):
        """Update the provided share with the provided state.

        :param share: either share object or text with its ID.
        :param state: text with new state to set for share.
        """
        return self._action(action_name, share, {"status": state})

    @api_versions.wraps("1.0", "2.6")
    def reset_state(self, share, state):
        return self._do_reset_state(share, state, "os-reset_status")

    @api_versions.wraps("2.7")  # noqa
    def reset_state(self, share, state):  # noqa
        return self._do_reset_state(share, state, "reset_status")

    def _do_extend(self, share, new_size, action_name):
        """Extend the size of the specified share.

        :param share: either share object or text with its ID.
        :param new_size: The desired size to extend share to.
        """
        return self._action(action_name, share, {"new_size": new_size})

    @api_versions.wraps("1.0", "2.6")
    def extend(self, share, new_size):
        return self._do_extend(share, new_size, "os-extend")

    @api_versions.wraps("2.7")  # noqa
    def extend(self, share, new_size):  # noqa
        return self._do_extend(share, new_size, "extend")

    def _do_shrink(self, share, new_size, action_name):
        """Shrink the size of the specified share.

        :param share: either share object or text with its ID.
        :param new_size: The desired size to shrink share to.
        """
        return self._action(action_name, share, {'new_size': new_size})

    @api_versions.wraps("1.0", "2.6")
    def shrink(self, share, new_size):
        return self._do_shrink(share, new_size, "os-shrink")

    @api_versions.wraps("2.7")  # noqa
    def shrink(self, share, new_size):   # noqa
        return self._do_shrink(share, new_size, "shrink")

    def list_instances(self, share):
        """List instances of the specified share.

        :param share: either share object or text with its ID.
        """
        return self._list(
            '/shares/%s/instances' % common_base.getid(share),
            'share_instances',
            obj_class=share_instances.ShareInstance
        )



# Copyright (c) 2015 Clinton Knight.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from manilaclient.tests.functional import base


class ManilaClientTestSchedulerStatsReadOnly(base.BaseTestCase):

    def test_pools_list(self):
        self.clients['admin'].manila('pool-list')

    def test_pools_list_with_debug_flag(self):
        self.clients['admin'].manila('pool-list', flags='--debug')

    def test_pools_list_with_detail(self):
        self.clients['admin'].manila('pool-list', params='--detail')

    def test_pools_list_with_share_type_filter(self):
        share_type = self.create_share_type(
            name=data_utils.rand_name('manilaclient_functional_test'),
            snapshot_support=True,
        )
        self.clients['admin'].manila('pool-list',
                                     params='--share_type ' +
                                            share_type['ID'])

    def test_pools_list_with_filters(self):
        self.clients['admin'].manila(
            'pool-list',
            params='--host myhost --backend mybackend --pool mypool')

    def test_pools_list_by_user(self):
        self.assertRaises(exceptions.CommandFailed,
                          self.clients['user'].manila,
                          'pool-list')



# Copyright 2010 Jacob Kaplan-Moss

# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base utilities to build API operation managers and objects on top of.
"""

import contextlib
import hashlib
import os

from manilaclient.common import cliutils
from manilaclient import exceptions
from manilaclient import utils


# Python 2.4 compat
try:
    all
except NameError:
    def all(iterable):
        return True not in (not x for x in iterable)


class Manager(utils.HookableMixin):
    """Manager for CRUD operations.

    Managers interact with a particular type of API (shares, snapshots,
    etc.) and provide CRUD operations for them.
    """
    resource_class = None

    def __init__(self, api):
        self.api = api
        self.client = api.client

    @property
    def api_version(self):
        return self.api.api_version

    def _list(self, url, response_key, obj_class=None, body=None):
        resp = None
        if body:
            resp, body = self.api.client.post(url, body=body)
        else:
            resp, body = self.api.client.get(url)

        if obj_class is None:
            obj_class = self.resource_class

        data = body[response_key]
        # NOTE(ja): keystone returns values as list as {'values': [ ... ]}
        #           unlike other services which just return the list...
        if isinstance(data, dict):
            try:
                data = data['values']
            except KeyError:
                pass

        with self.completion_cache('human_id', obj_class, mode="w"):
            with self.completion_cache('uuid', obj_class, mode="w"):
                resource = [obj_class(self, res, loaded=True)
                            for res in data if res]
                if 'count' in body:
                    return resource, body['count']
                else:
                    return resource

    @contextlib.contextmanager
    def completion_cache(self, cache_type, obj_class, mode):
        """Bash autocompletion items storage.

        The completion cache store items that can be used for bash
        autocompletion, like UUIDs or human-friendly IDs.

        A resource listing will clear and repopulate the cache.

        A resource create will append to the cache.

        Delete is not handled because listings are assumed to be performed
        often enough to keep the cache reasonably up-to-date.
        """
        base_dir = cliutils.env('manilaclient_UUID_CACHE_DIR',
                                'MANILACLIENT_UUID_CACHE_DIR',
                                default="~/.cache/manilaclient")

        # NOTE(sirp): Keep separate UUID caches for each username + endpoint
        # pair
        username = cliutils.env('OS_USERNAME', 'MANILA_USERNAME')
        url = cliutils.env('OS_URL', 'MANILA_URL')
        uniqifier = hashlib.sha1(username.encode('utf-8') +
                                 url.encode('utf-8')).hexdigest()

        cache_dir = os.path.expanduser(os.path.join(base_dir, uniqifier))

        try:
            os.makedirs(cache_dir, 0o755)
        except OSError:
            # NOTE(kiall): This is typically either permission denied while
            #              attempting to create the directory, or the directory
            #              already exists. Either way, don't fail.
            pass

        resource = obj_class.__name__.lower()
        filename = "%s-%s-cache" % (resource, cache_type.replace('_', '-'))
        path = os.path.join(cache_dir, filename)

        cache_attr = "_%s_cache" % cache_type

        try:
            setattr(self, cache_attr, open(path, mode))
        except IOError:
            # NOTE(kiall): This is typically a permission denied while
            #              attempting to write the cache file.
            pass

        try:
            yield
        finally:
            cache = getattr(self, cache_attr, None)
            if cache:
                cache.close()
                delattr(self, cache_attr)

    def write_to_completion_cache(self, cache_type, val):
        cache = getattr(self, "_%s_cache" % cache_type, None)
        if cache:
            try:
                cache.write("%s\n" % val)
            except UnicodeEncodeError:
                pass

    def _get(self, url, response_key=None):
        resp, body = self.api.client.get(url)
        if response_key:
            return self.resource_class(self, body[response_key], loaded=True)
        else:
            return self.resource_class(self, body, loaded=True)

    def _get_with_base_url(self, url, response_key=None):
        resp, body = self.api.client.get_with_base_url(url)
        if response_key:
            return [self.resource_class(self, res, loaded=True)
                    for res in body[response_key] if res]
        else:
            return self.resource_class(self, body, loaded=True)

    def _create(self, url, body, response_key, return_raw=False, **kwargs):
        self.run_hooks('modify_body_for_create', body, **kwargs)
        resp, body = self.api.client.post(url, body=body)
        if return_raw:
            return body[response_key]

        with self.completion_cache('human_id', self.resource_class, mode="a"):
            with self.completion_cache('uuid', self.resource_class, mode="a"):
                return self.resource_class(self, body[response_key])

    def _delete(self, url):
        resp, body = self.api.client.delete(url)

    def _update(self, url, body, response_key=None, **kwargs):
        self.run_hooks('modify_body_for_update', body, **kwargs)
        resp, body = self.api.client.put(url, body=body)
        if body:
            if response_key:
                return self.resource_class(self, body[response_key])
            else:
                return self.resource_class(self, body)

    def _build_query_string(self, search_opts):
        search_opts = search_opts or {}
        params = sorted(
            [(k, v) for (k, v) in search_opts.items() if v])
        query_string = "?%s" % utils.safe_urlencode(params) if params else ''
        return query_string


class ManagerWithFind(Manager):
    """Like a `Manager`, but with additional `find()`/`findall()` methods."""
    def find(self, **kwargs):
        """Find a single item with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        matches = self.findall(**kwargs)
        num_matches = len(matches)
        if num_matches == 0:
            msg = "No %s matching %s." % (self.resource_class.__name__, kwargs)
            raise exceptions.NotFound(404, msg)
        elif num_matches > 1:
            raise exceptions.NoUniqueMatch
        else:
            return matches[0]

    def findall(self, **kwargs):
        """Find all items with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        found = []
        searches = list(kwargs.items())

        search_opts = {'all_tenants': 1}
        for obj in self.list(search_opts=search_opts):
            try:
                if all(getattr(obj, attr) == value
                       for (attr, value) in searches):
                    found.append(obj)
            except AttributeError:
                continue

        return found

    def list(self, search_opts=None):
        raise NotImplementedError



# Copyright (c) 2011 Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


"""
Share Type interface.
"""

from manilaclient import api_versions
from manilaclient import base
from manilaclient.common.apiclient import base as common_base
from manilaclient import exceptions


class ShareType(common_base.Resource):
    """A Share Type is the type of share to be created."""

    def __init__(self, manager, info, loaded=False):
        super(ShareType, self).__init__(manager, info, loaded)
        self._required_extra_specs = info.get('required_extra_specs', {})
        self._optional_extra_specs = info.get('extra_specs', {}).copy()
        for key in self._required_extra_specs.keys():
            self._optional_extra_specs.pop(key, None)

    def __repr__(self):
        return "<ShareType: %s>" % self.name

    @property
    def is_public(self):
        """Provide a user-friendly accessor to [os-]share-type-access."""
        return self._info.get(
            "share_type_access:is_public",
            self._info.get("os-share-type-access:is_public", "N/A"))

    def get_keys(self, prefer_resource_data=True):
        """Get extra specs from a share type.

        :param prefer_resource_data: By default extra_specs are retrieved from
        resource data, but user can force this method to make API call.
        :return: dict with extra specs
        """
        extra_specs = getattr(self, 'extra_specs', None)

        if prefer_resource_data and extra_specs:
            return extra_specs

        _resp, body = self.manager.api.client.get(
            "/types/%s/extra_specs" % common_base.getid(self))

        self.extra_specs = body["extra_specs"]

        return body["extra_specs"]

    def get_required_keys(self):
        return self._required_extra_specs

    def get_optional_keys(self):
        return self._optional_extra_specs

    def set_keys(self, metadata):
        """Set extra specs on a share type.

        :param type : The :class:`ShareType` to set extra spec on
        :param metadata: A dict of key/value pairs to be set
        """
        body = {'extra_specs': metadata}
        return self.manager._create(
            "/types/%s/extra_specs" % common_base.getid(self),
            body,
            "extra_specs",
            return_raw=True,
        )

    def unset_keys(self, keys):
        """Unset extra specs on a share type.

        :param type_id: The :class:`ShareType` to unset extra spec on
        :param keys: A list of keys to be unset
        """

        # NOTE(jdg): This wasn't actually doing all of the keys before
        # the return in the loop resulted in ony ONE key being unset.
        # since on success the return was NONE, we'll only interrupt the loop
        # and return if there's an error
        resp = None
        for k in keys:
            resp = self.manager._delete(
                "/types/%s/extra_specs/%s" % (common_base.getid(self), k))
            if resp is not None:
                return resp

    def update(self, **kwargs):
        """Update this share type."""
        return self.manager.update(self, **kwargs)


class ShareTypeManager(base.ManagerWithFind):
    """Manage :class:`ShareType` resources."""

    resource_class = ShareType

    def list(self, search_opts=None, show_all=True):
        """Get a list of all share types.

        :rtype: list of :class:`ShareType`.
        """
        search_opts = search_opts or {}
        if show_all:
            search_opts['is_public'] = 'all'
        query_string = self._build_query_string(search_opts)
        return self._list("/types%s" % query_string, "share_types")

    def show(self, share_type):
        """Get a share.

        :param share: either share object or text with its ID.
        :rtype: :class:`Share`
        """
        type_id = common_base.getid(share_type)
        return self._get("/types/%s" % type_id, "share_type")

    def get(self, share_type="default"):
        """Get a specific share type.

        :param share_type: The ID of the :class:`ShareType` to get.
        :rtype: :class:`ShareType`
        """
        return self._get("/types/%s" % common_base.getid(share_type),
                         "share_type")

    def delete(self, share_type):
        """Delete a specific share_type.

        :param share_type: The name or ID of the :class:`ShareType` to get.
        """
        self._delete("/types/%s" % common_base.getid(share_type))

    def _do_create(self, name, extra_specs, is_public,
                   is_public_keyname="share_type_access:is_public",
                   description=None):
        """Create a share type.

        :param name: Descriptive name of the share type
        :rtype: :class:`ShareType`
        """

        body = {
            "share_type": {
                "name": name,
                is_public_keyname: is_public,
                "extra_specs": extra_specs,
            }
        }

        if description:
            body["share_type"]["description"] = description
        return self._create("/types", body, "share_type")

    def _do_update(self, share_type, name=None, is_public=None,
                   is_public_keyname="share_type_access:is_public",
                   description=None):
        """Update the name and/or description for a share type.

        :param share_type: the ID of the :class: `ShareType` to update.
        :param name: Descriptive name of the share type.
        :param description: Description of the share type.
        :rtype: :class:`ShareType`
        """

        body = {
            "share_type": {}
        }

        if name:
            body["share_type"]["name"] = name
        if is_public is not None:
            body["share_type"][is_public_keyname] = is_public
        if description or description == "":
            body["share_type"]["description"] = description
        return self._update("/types/%s" % common_base.getid(share_type),
                            body, "share_type")

    @api_versions.wraps("1.0", "2.6")
    def create(self, name, spec_driver_handles_share_servers,
               spec_snapshot_support=None, is_public=True, extra_specs=None):

        if extra_specs is None:
            extra_specs = {}

        self._handle_spec_driver_handles_share_servers(
            extra_specs, spec_driver_handles_share_servers)
        self._handle_spec_snapshot_support(
            extra_specs, spec_snapshot_support, set_default=True)

        return self._do_create(
            name, extra_specs, is_public,
            is_public_keyname="os-share-type-access:is_public")

    @api_versions.wraps("2.7", "2.23")  # noqa
    def create(self, name, spec_driver_handles_share_servers,  # noqa
               spec_snapshot_support=None, is_public=True, extra_specs=None):

        if extra_specs is None:
            extra_specs = {}

        self._handle_spec_driver_handles_share_servers(
            extra_specs, spec_driver_handles_share_servers)
        self._handle_spec_snapshot_support(
            extra_specs, spec_snapshot_support, set_default=True)

        return self._do_create(name, extra_specs, is_public)

    @api_versions.wraps("2.24", "2.40")  # noqa
    def create(self, name, spec_driver_handles_share_servers,  # noqa
               spec_snapshot_support=None, is_public=True, extra_specs=None):

        if extra_specs is None:
            extra_specs = {}

        self._handle_spec_driver_handles_share_servers(
            extra_specs, spec_driver_handles_share_servers)
        self._handle_spec_snapshot_support(extra_specs, spec_snapshot_support)

        return self._do_create(name, extra_specs, is_public)

    @api_versions.wraps("2.41")  # noqa
    def create(self, name, spec_driver_handles_share_servers,  # noqa
               spec_snapshot_support=None, is_public=True, extra_specs=None,
               description=None):
        if extra_specs is None:
            extra_specs = {}

        self._handle_spec_driver_handles_share_servers(
            extra_specs, spec_driver_handles_share_servers)
        self._handle_spec_snapshot_support(extra_specs, spec_snapshot_support)

        return self._do_create(name, extra_specs, is_public,
                               description=description)

    @api_versions.wraps("2.50")
    def update(self, share_type, name=None, is_public=None, description=None):
        return self._do_update(share_type, name, is_public,
                               description=description)

    def _handle_spec_driver_handles_share_servers(
            self, extra_specs, spec_driver_handles_share_servers):
        """Validation and default for DHSS extra spec."""

        if spec_driver_handles_share_servers is not None:
            if 'driver_handles_share_servers' in extra_specs:
                msg = ("'driver_handles_share_servers' is already set via "
                       "positional argument.")
                raise exceptions.CommandError(msg)
            else:
                extra_specs['driver_handles_share_servers'] = (
                    spec_driver_handles_share_servers)
        else:
            msg = ("'driver_handles_share_servers' is not set via "
                   "positional argument.")
            raise exceptions.CommandError(msg)

    def _handle_spec_snapshot_support(self, extra_specs, spec_snapshot_support,
                                      set_default=False):
        """Validation and default for snapshot extra spec."""

        if spec_snapshot_support is not None:
            if 'snapshot_support' in extra_specs:
                msg = "'snapshot_support' extra spec is provided twice."
                raise exceptions.CommandError(msg)
            else:
                extra_specs['snapshot_support'] = spec_snapshot_support
        elif 'snapshot_support' not in extra_specs and set_default:
            extra_specs['snapshot_support'] = True



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import mock

from osc_lib import exceptions

from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes

from manilaclient import api_versions
from manilaclient.common.apiclient.exceptions import BadRequest
from manilaclient.osc.v2 import quotas as osc_quotas
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes


class TestQuotas(manila_fakes.TestShare):

    def setUp(self):
        super(TestQuotas, self).setUp()

        self.quotas_mock = self.app.client_manager.share.quotas
        self.quotas_mock.reset_mock()

        self.app.client_manager.share.api_version = api_versions.APIVersion(
            api_versions.MAX_VERSION
        )


class TestQuotaSet(TestQuotas):
    project = identity_fakes.FakeProject.create_one_project()
    user = identity_fakes.FakeUser.create_one_user()

    def setUp(self):
        super(TestQuotaSet, self).setUp()

        self.quotas = manila_fakes.FakeQuotaSet.create_fake_quotas()
        self.quotas_mock.update = mock.Mock()
        self.quotas_mock.update.return_value = None

        self.cmd = osc_quotas.QuotaSet(self.app, None)

    def test_quota_set_shares(self):
        arglist = [
            '--project', self.project.id,
            '--shares', '40'
        ]
        verifylist = [
            ('project', self.project.id),
            ('shares', 40)
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            result = self.cmd.take_action(parsed_args)
            self.quotas_mock.update.assert_called_with(
                force=None,
                gigabytes=None,
                share_networks=None,
                shares=40,
                snapshot_gigabytes=None,
                snapshots=None,
                per_share_gigabytes=None,
                tenant_id=self.project.id,
                user_id=None)
            self.assertIsNone(result)

    def test_quota_set_gigabytes(self):
        arglist = [
            '--project', self.project.id,
            '--gigabytes', '1100'
        ]
        verifylist = [
            ('project', self.project.id),
            ('gigabytes', 1100)
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            result = self.cmd.take_action(parsed_args)
            self.quotas_mock.update.assert_called_with(
                force=None,
                gigabytes=1100,
                share_networks=None,
                shares=None,
                snapshot_gigabytes=None,
                snapshots=None,
                per_share_gigabytes=None,
                tenant_id=self.project.id,
                user_id=None)
            self.assertIsNone(result)

    def test_quota_set_share_type(self):
        arglist = [
            '--project', self.project.id,
            '--share-type', 'default'
        ]
        verifylist = [
            ('project', self.project.id),
            ('share_type', 'default')
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            result = self.cmd.take_action(parsed_args)
            self.quotas_mock.update.assert_called_with(
                force=None,
                gigabytes=None,
                share_networks=None,
                share_type='default',
                shares=None,
                snapshot_gigabytes=None,
                snapshots=None,
                per_share_gigabytes=None,
                tenant_id=self.project.id,
                user_id=None)
            self.assertIsNone(result)

    def test_quota_set_force(self):
        arglist = [
            '--project', self.project.id,
            '--force',
            '--shares', '40'
        ]
        verifylist = [
            ('project', self.project.id),
            ('force', True),
            ('shares', 40)
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            result = self.cmd.take_action(parsed_args)
            self.quotas_mock.update.assert_called_with(
                force=True,
                gigabytes=None,
                share_networks=None,
                shares=40,
                snapshot_gigabytes=None,
                snapshots=None,
                tenant_id=self.project.id,
                per_share_gigabytes=None,
                user_id=None)
            self.assertIsNone(result)

    def test_quota_set_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            '2.39'
        )

        arglist = [
            '--project', self.project.id,
            '--share-groups', '40'
        ]
        verifylist = [
            ('project', self.project.id),
            ('share_groups', 40)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_quota_set_update_exception(self):
        arglist = [
            '--project', self.project.id,
            '--share-groups', '40',
            '--share-group-snapshots', '40'
        ]
        verifylist = [
            ('project', self.project.id),
            ('share_groups', 40),
            ('share_group_snapshots', 40)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.quotas_mock.update.side_effect = BadRequest()
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_quota_set_nothing_to_set_exception(self):
        arglist = [
            '--project', self.project.id,
        ]
        verifylist = [
            ('project', self.project.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_quota_set_share_replicas(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            '2.53'
        )

        arglist = [
            '--project', self.project.id,
            '--share-replicas', '2',
        ]
        verifylist = [
            ('project', self.project.id),
            ('share_replicas', 2)
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            result = self.cmd.take_action(parsed_args)
            self.quotas_mock.update.assert_called_with(
                force=None,
                gigabytes=None,
                share_networks=None,
                share_replicas=2,
                shares=None,
                snapshot_gigabytes=None,
                snapshots=None,
                per_share_gigabytes=None,
                tenant_id=self.project.id,
                user_id=None)
            self.assertIsNone(result)

    def test_quota_set_replica_gigabytes_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            '2.51')
        arglist = [
            '--project', self.project.id,
            '--replica-gigabytes', '10',
        ]
        verifylist = [
            ('project', self.project.id),
            ('replica_gigabytes', 10)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_quota_set_per_share_gigabytes(self):
        arglist = [
            '--project', self.project.id,
            '--per-share-gigabytes', '10',
        ]
        verifylist = [
            ('project', self.project.id),
            ('per_share_gigabytes', 10)
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            result = self.cmd.take_action(parsed_args)
            self.quotas_mock.update.assert_called_with(
                force=None,
                gigabytes=None,
                share_networks=None,
                shares=None,
                snapshot_gigabytes=None,
                snapshots=None,
                per_share_gigabytes=10,
                tenant_id=self.project.id,
                user_id=None)
            self.assertIsNone(result)


class TestQuotaShow(TestQuotas):
    project = identity_fakes.FakeProject.create_one_project()
    user = identity_fakes.FakeUser.create_one_user()

    def setUp(self):
        super(TestQuotaShow, self).setUp()

        self.quotas = manila_fakes.FakeQuotaSet.create_fake_quotas()
        self.quotas_mock.get.return_value = self.quotas
        self.cmd = osc_quotas.QuotaShow(self.app, None)

    def test_quota_show(self):
        arglist = [
            '--project', self.project.id
        ]
        verifylist = [
            ('project', self.project.id)
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
            columns, data = self.cmd.take_action(parsed_args)

            self.quotas_mock.get.assert_called_with(
                detail=False,
                tenant_id=self.project.id,
                user_id=None
            )

            self.assertCountEqual(columns, self.quotas.keys())
            self.assertCountEqual(data, self.quotas._info.values())

    def test_quota_show_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            '2.38'
        )

        arglist = [
            '--project', self.project.id,
            '--share-type', 'default'
        ]
        verifylist = [
            ('project', self.project.id),
            ('share_type', 'default')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_quota_show_user_id_share_type_exception(self):
        arglist = [
            '--project', self.project.id,
            '--share-type', 'default',
            '--user', self.user.id
        ]
        verifylist = [
            ('project', self.project.id),
            ('share_type', 'default'),
            ('user', self.user.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_quota_show_defaults(self):
        arglist = [
            '--project', self.project.id,
            '--defaults'
        ]
        verifylist = [
            ('project', self.project.id),
            ('defaults', True)
        ]

        self.quotas_mock.defaults = mock.Mock()
        self.quotas_mock.defaults.return_value = self.quotas

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
            columns, data = self.cmd.take_action(parsed_args)

            self.quotas_mock.defaults.assert_called_with(self.project.id)

            self.assertCountEqual(columns, self.quotas.keys())
            self.assertCountEqual(data, self.quotas._info.values())


class TestQuotaDelete(TestQuotas):
    project = identity_fakes.FakeProject.create_one_project()
    user = identity_fakes.FakeUser.create_one_user()

    def setUp(self):
        super(TestQuotaDelete, self).setUp()

        self.quotas = manila_fakes.FakeQuotaSet.create_fake_quotas()
        self.quotas_mock.delete.return_value = None
        self.cmd = osc_quotas.QuotaDelete(self.app, None)

    def test_quota_delete(self):
        arglist = [
            '--project', self.project.id
        ]
        verifylist = [
            ('project', self.project.id)
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            result = self.cmd.take_action(parsed_args)
            self.quotas_mock.delete.assert_called_with(
                tenant_id=self.project.id,
                user_id=None)
            self.assertIsNone(result)

    def test_quota_delete_share_type(self):
        arglist = [
            '--project', self.project.id,
            '--share-type', 'default'
        ]
        verifylist = [
            ('project', self.project.id),
            ('share_type', 'default')
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            result = self.cmd.take_action(parsed_args)
            self.quotas_mock.delete.assert_called_with(
                share_type='default',
                tenant_id=self.project.id,
                user_id=None)
            self.assertIsNone(result)

    def test_quota_delete_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            '2.38'
        )

        arglist = [
            '--project', self.project.id,
            '--share-type', 'default'
        ]
        verifylist = [
            ('project', self.project.id),
            ('share_type', 'default')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_quota_delete_user_share_type_exeption(self):
        arglist = [
            '--project', self.project.id,
            '--share-type', 'default',
            '--user', self.user.id
        ]
        verifylist = [
            ('project', self.project.id),
            ('share_type', 'default'),
            ('user', self.user.id)
        ]

        with mock.patch('osc_lib.utils.find_resource') as mock_find_resource:
            mock_find_resource.return_value = self.project

            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
            self.assertRaises(
                exceptions.CommandError, self.cmd.take_action, parsed_args)



# Copyright 2017 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes as fake
from manilaclient.v2 import messages


class MessageTest(utils.TestCase):

    def setUp(self):
        super(MessageTest, self).setUp()
        self.manager = messages.MessageManager(fake.FakeClient())
        self.message = messages.Message(
            self.manager, {'id': 'fake_id'})
        self.fake_kwargs = {'key': 'value'}

    def test_repr(self):
        result = str(self.message)

        self.assertEqual('<Message: fake_id>', result)

    def test_delete(self):
        mock_manager_delete = self.mock_object(self.manager, 'delete')

        self.message.delete()

        mock_manager_delete.assert_called_once_with(self.message)


@ddt.ddt
class MessageManagerTest(utils.TestCase):

    def setUp(self):
        super(MessageManagerTest, self).setUp()
        self.manager = messages.MessageManager(fake.FakeClient())

    def test_get(self):
        fake_message = fake.Message()
        mock_get = self.mock_object(
            self.manager, '_get', mock.Mock(return_value=fake_message))

        result = self.manager.get(fake.Message.id)

        self.assertIs(fake_message, result)
        mock_get.assert_called_once_with(
            messages.RESOURCE_PATH % fake.Message.id,
            messages.RESOURCE_NAME)

    def test_list(self):
        fake_message = fake.Message()
        mock_list = self.mock_object(
            self.manager, '_list', mock.Mock(return_value=[fake_message]))

        result = self.manager.list()

        self.assertEqual([fake_message], result)
        mock_list.assert_called_once_with(
            messages.RESOURCES_PATH,
            messages.RESOURCES_NAME)

    @ddt.data(
        ({'action_id': 1, 'resource_type': 'share'},
         '?action_id=1&resource_type=share'),
        ({'action_id': 1}, '?action_id=1'),
    )
    @ddt.unpack
    def test_list_with_filters(self, filters, filters_path):
        fake_message = fake.Message()
        mock_list = self.mock_object(
            self.manager, '_list', mock.Mock(return_value=[fake_message]))

        result = self.manager.list(search_opts=filters)

        self.assertEqual([fake_message], result)
        expected_path = (messages.RESOURCES_PATH + filters_path)
        mock_list.assert_called_once_with(
            expected_path, messages.RESOURCES_NAME)

    @ddt.data('id', 'project_id', 'request_id', 'resource_type', 'action_id',
              'detail_id', 'resource_id', 'message_level', 'expires_at',
              'request_id', 'created_at')
    def test_list_with_sorting(self, key):
        fake_message = fake.Message()
        mock_list = self.mock_object(
            self.manager, '_list', mock.Mock(return_value=[fake_message]))

        result = self.manager.list(sort_dir='asc', sort_key=key)

        self.assertEqual([fake_message], result)
        expected_path = (
            messages.RESOURCES_PATH + '?sort_dir=asc&sort_key=' +
            key)
        mock_list.assert_called_once_with(
            expected_path, messages.RESOURCES_NAME)

    @ddt.data(
        ('name', 'invalid'),
        ('invalid', 'asc'),
    )
    @ddt.unpack
    def test_list_with_invalid_sorting(self, sort_key, sort_dir):
        self.assertRaises(
            ValueError,
            self.manager.list, sort_dir=sort_dir, sort_key=sort_key)

    def test_delete(self):
        mock_delete = self.mock_object(self.manager, '_delete')
        mock_post = self.mock_object(self.manager.api.client, 'post')

        self.manager.delete(fake.Message())

        mock_delete.assert_called_once_with(
            messages.RESOURCE_PATH % fake.Message.id)
        self.assertFalse(mock_post.called)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from unittest import mock

from osc_lib import exceptions
from osc_lib import utils as oscutils

from manilaclient import api_versions
from manilaclient.common.apiclient.exceptions import BadRequest
from manilaclient.common.apiclient.exceptions import NotFound
from manilaclient.osc.v2 import share_types as osc_share_types
from manilaclient.tests.unit.osc import osc_utils
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes

COLUMNS = [
    'id',
    'name',
    'visibility',
    'is_default',
    'required_extra_specs',
    'optional_extra_specs',
    'description'
]


class TestShareType(manila_fakes.TestShare):

    def setUp(self):
        super(TestShareType, self).setUp()

        self.shares_mock = self.app.client_manager.share.share_types
        self.shares_mock.reset_mock()
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            api_versions.MAX_VERSION)


class TestShareTypeCreate(TestShareType):

    def setUp(self):
        super(TestShareTypeCreate, self).setUp()

        self.new_share_type = manila_fakes.FakeShareType.create_one_sharetype()
        self.shares_mock.create.return_value = self.new_share_type

        # Get the command object to test
        self.cmd = osc_share_types.CreateShareType(self.app, None)

        self.data = [
            self.new_share_type.id,
            self.new_share_type.name,
            'public',
            self.new_share_type.is_default,
            'driver_handles_share_servers : True',
            ('replication_type : readable\n'
             'mount_snapshot_support : False\n'
             'revert_to_snapshot_support : False\n'
             'create_share_from_snapshot_support : True\n'
             'snapshot_support : True'),
            self.new_share_type.description,
        ]

    def test_share_type_create_required_args(self):
        """Verifies required arguments."""

        arglist = [
            self.new_share_type.name,
            'True'
        ]
        verifylist = [
            ('name', self.new_share_type.name),
            ('spec_driver_handles_share_servers', 'True')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.shares_mock.create.assert_called_with(
            extra_specs={},
            is_public=True,
            name=self.new_share_type.name,
            spec_driver_handles_share_servers=True
        )

        self.assertCountEqual(COLUMNS, columns)
        self.assertCountEqual(self.data, data)

    def test_share_type_create_missing_required_arg(self):
        """Verifies missing required arguments."""

        arglist = [
            self.new_share_type.name
        ]
        verifylist = [
            ('name', self.new_share_type.name)
        ]

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_type_create_private(self):
        arglist = [
            self.new_share_type.name,
            'True',
            '--public', 'False'
        ]
        verifylist = [
            ('name', self.new_share_type.name),
            ('spec_driver_handles_share_servers', 'True'),
            ('public', 'False')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.shares_mock.create.assert_called_with(
            extra_specs={},
            is_public=False,
            name=self.new_share_type.name,
            spec_driver_handles_share_servers=True
        )

        self.assertCountEqual(COLUMNS, columns)
        self.assertCountEqual(self.data, data)

    def test_share_type_create_extra_specs(self):
        arglist = [
            self.new_share_type.name,
            'True',
            '--extra-specs', 'snapshot_support=true'
        ]
        verifylist = [
            ('name', self.new_share_type.name),
            ('spec_driver_handles_share_servers', 'True'),
            ('extra_specs', ['snapshot_support=true'])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.shares_mock.create.assert_called_with(
            extra_specs={'snapshot_support': 'True'},
            is_public=True,
            name=self.new_share_type.name,
            spec_driver_handles_share_servers=True
        )

        self.assertCountEqual(COLUMNS, columns)
        self.assertCountEqual(self.data, data)

    def test_share_type_create_dhss_invalid_value(self):
        arglist = [
            self.new_share_type.name,
            'non_bool_value'
        ]
        verifylist = [
            ('name', self.new_share_type.name),
            ('spec_driver_handles_share_servers', 'non_bool_value')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_type_create_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.40")

        arglist = [
            self.new_share_type.name,
            'True',
            '--description', 'Description'
        ]
        verifylist = [
            ('name', self.new_share_type.name),
            ('spec_driver_handles_share_servers', 'True'),
            ('description', 'Description')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_type_create_dhss_defined_twice(self):
        arglist = [
            self.new_share_type.name,
            'True',
            '--extra-specs', 'driver_handles_share_servers=true'
        ]
        verifylist = [
            ('name', self.new_share_type.name),
            ('spec_driver_handles_share_servers', 'True'),
            ('extra_specs', ['driver_handles_share_servers=true'])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_type_create_bool_args(self):
        arglist = [
            self.new_share_type.name,
            'True',
            '--snapshot-support', 'true'
        ]
        verifylist = [
            ('name', self.new_share_type.name),
            ('spec_driver_handles_share_servers', 'True'),
            ('snapshot_support', 'true')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.shares_mock.create.assert_called_with(
            extra_specs={'snapshot_support': 'True'},
            is_public=True,
            name=self.new_share_type.name,
            spec_driver_handles_share_servers=True
        )

        self.assertCountEqual(COLUMNS, columns)
        self.assertCountEqual(self.data, data)


class TestShareTypeDelete(TestShareType):

    share_types = manila_fakes.FakeShareType.create_share_types(count=2)

    def setUp(self):
        super(TestShareTypeDelete, self).setUp()

        self.shares_mock.get = manila_fakes.FakeShareType.get_share_types(
            self.share_types)
        self.shares_mock.delete.return_value = None

        # Get the command object to test
        self.cmd = osc_share_types.DeleteShareType(self.app, None)

    def test_share_type_delete_one(self):
        arglist = [
            self.share_types[0].id
        ]

        verifylist = [
            ('share_types', [self.share_types[0].id])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.shares_mock.delete.assert_called_with(self.share_types[0])
        self.assertIsNone(result)

    def test_share_type_delete_multiple(self):
        arglist = []
        for t in self.share_types:
            arglist.append(t.id)
        verifylist = [
            ('share_types', arglist),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        calls = []
        for t in self.share_types:
            calls.append(mock.call(t))
        self.shares_mock.delete.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_delete_share_type_with_exception(self):
        arglist = [
            'non_existing_type',
        ]
        verifylist = [
            ('share_types', arglist),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.shares_mock.delete.side_effect = exceptions.CommandError()
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestShareTypeSet(TestShareType):

    def setUp(self):
        super(TestShareTypeSet, self).setUp()

        self.share_type = manila_fakes.FakeShareType.create_one_sharetype(
            methods={'set_keys': None, 'update': None})
        self.shares_mock.get.return_value = self.share_type

        # Get the command object to test
        self.cmd = osc_share_types.SetShareType(self.app, None)

    def test_share_type_set_extra_specs(self):
        arglist = [
            self.share_type.id,
            '--extra-specs', 'snapshot_support=true'
        ]
        verifylist = [
            ('share_type', self.share_type.id),
            ('extra_specs', ['snapshot_support=true'])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.share_type.set_keys.assert_called_with(
            {'snapshot_support': 'True'})
        self.assertIsNone(result)

    def test_share_type_set_name(self):
        arglist = [
            self.share_type.id,
            '--name', 'new name'
        ]
        verifylist = [
            ('share_type', self.share_type.id),
            ('name', 'new name')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.share_type.update.assert_called_with(
            name='new name')
        self.assertIsNone(result)

    def test_share_type_set_description(self):
        arglist = [
            self.share_type.id,
            '--description', 'new description'
        ]
        verifylist = [
            ('share_type', self.share_type.id),
            ('description', 'new description')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.share_type.update.assert_called_with(
            description='new description')
        self.assertIsNone(result)

    def test_share_type_set_visibility(self):
        arglist = [
            self.share_type.id,
            '--public', 'false'
        ]
        verifylist = [
            ('share_type', self.share_type.id),
            ('public', 'false')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.share_type.update.assert_called_with(
            is_public=False)
        self.assertIsNone(result)

    def test_share_type_set_extra_specs_exception(self):
        arglist = [
            self.share_type.id,
            '--extra-specs', 'snapshot_support=true'
        ]
        verifylist = [
            ('share_type', self.share_type.id),
            ('extra_specs', ['snapshot_support=true'])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.share_type.set_keys.side_effect = BadRequest()
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_type_set_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.49")

        arglist = [
            self.share_type.id,
            '--name', 'new name',
        ]
        verifylist = [
            ('share_type', self.share_type.id),
            ('name', 'new name'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestShareTypeUnset(TestShareType):

    def setUp(self):
        super(TestShareTypeUnset, self).setUp()

        self.share_type = manila_fakes.FakeShareType.create_one_sharetype(
            methods={'unset_keys': None})
        self.shares_mock.get.return_value = self.share_type

        # Get the command object to test
        self.cmd = osc_share_types.UnsetShareType(self.app, None)

    def test_share_type_unset_extra_specs(self):
        arglist = [
            self.share_type.id,
            'snapshot_support'
        ]
        verifylist = [
            ('share_type', self.share_type.id),
            ('extra_specs', ['snapshot_support'])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.share_type.unset_keys.assert_called_with(['snapshot_support'])
        self.assertIsNone(result)

    def test_share_type_unset_exception(self):
        arglist = [
            self.share_type.id,
            'snapshot_support'
        ]
        verifylist = [
            ('share_type', self.share_type.id),
            ('extra_specs', ['snapshot_support'])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.share_type.unset_keys.side_effect = NotFound()
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestShareTypeList(TestShareType):

    share_types = manila_fakes.FakeShareType.create_share_types()

    def setUp(self):
        super(TestShareTypeList, self).setUp()

        self.shares_mock.list.return_value = self.share_types

        # Get the command object to test
        self.cmd = osc_share_types.ListShareType(self.app, None)

        self.values = (oscutils.get_dict_properties(
            s._info, COLUMNS) for s in self.share_types)

    def test_share_type_list_no_options(self):
        arglist = []
        verifylist = [
            ('all', False)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.shares_mock.list.assert_called_once_with(
            search_opts={},
            show_all=False
        )
        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(self.values), list(data))

    def test_share_type_list_all(self):
        arglist = [
            '--all',
        ]
        verifylist = [
            ('all', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.shares_mock.list.assert_called_once_with(
            search_opts={},
            show_all=True)
        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(self.values), list(data))

    def test_share_type_list_extra_specs(self):
        arglist = [
            '--extra-specs', 'snapshot_support=true'
        ]
        verifylist = [
            ('extra_specs', ['snapshot_support=true'])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.shares_mock.list.assert_called_once_with(
            search_opts={'extra_specs': {'snapshot_support': 'True'}},
            show_all=False)
        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(self.values), list(data))

    def test_share_type_list_api_versions_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.42")

        arglist = [
            '--extra-specs', 'snapshot_support=true'
        ]
        verifylist = [
            ('extra_specs', ['snapshot_support=true'])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestShareTypeShow(TestShareType):

    def setUp(self):
        super(TestShareTypeShow, self).setUp()

        self.share_type = manila_fakes.FakeShareType.create_one_sharetype()

        self.shares_mock.get.return_value = self.share_type

        # Get the command object to test
        self.cmd = osc_share_types.ShowShareType(self.app, None)

        self.data = [
            self.share_type.id,
            self.share_type.name,
            'public',
            self.share_type.is_default,
            'driver_handles_share_servers : True',
            ('replication_type : readable\n'
             'mount_snapshot_support : False\n'
             'revert_to_snapshot_support : False\n'
             'create_share_from_snapshot_support : True\n'
             'snapshot_support : True'),
            self.share_type.description,
        ]

    def test_share_type_show(self):
        arglist = [
            self.share_type.id
        ]
        verifylist = [
            ("share_type", self.share_type.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.shares_mock.get.assert_called_with(self.share_type.id)

        self.assertCountEqual(COLUMNS, columns)
        self.assertCountEqual(self.data, data)



#   Copyright 2021 Red Hat Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from osc_lib import exceptions as osc_exceptions
from osc_lib import utils as osc_lib_utils

from manilaclient.common.apiclient import exceptions as api_exceptions
from manilaclient.common import cliutils
from manilaclient.osc.v2 import (
    share_snapshot_instances as osc_share_snapshot_instances)
from manilaclient.tests.unit.osc import osc_utils
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes

COLUMNS = ['ID', 'Snapshot ID', 'Status']

COLUMNS_DETAIL = [
    'ID',
    'Snapshot ID',
    'Status',
    'Created At',
    'Updated At',
    'Share ID',
    'Share Instance ID',
    'Progress',
    'Provider Location'
]


class TestShareSnapshotInstance(manila_fakes.TestShare):

    def setUp(self):
        super(TestShareSnapshotInstance, self).setUp()

        self.share_snapshots_mock = (
            self.app.client_manager.share.share_snapshots)
        self.share_snapshots_mock.reset_mock()

        self.share_snapshot_instances_mock = (
            self.app.client_manager.share.share_snapshot_instances)
        self.share_snapshot_instances_mock.reset_mock()

        self.share_snapshot_instances_el_mock = (
            self.app.client_manager
                .share.share_snapshot_instance_export_locations)
        self.share_snapshot_instances_el_mock.reset_mock()


class TestShareSnapshotInstanceList(TestShareSnapshotInstance):

    def setUp(self):
        super(TestShareSnapshotInstanceList, self).setUp()

        self.share_snapshot_instances = (
            manila_fakes.FakeShareSnapshotIntances
                        .create_share_snapshot_instances(count=2))

        self.share_snapshot_instances_mock.list.return_value = (
            self.share_snapshot_instances)

        self.cmd = (
            osc_share_snapshot_instances.ListShareSnapshotInstance(self.app,
                                                                   None))

    def test_share_snapshot_instance_list(self):
        values = (osc_lib_utils.get_dict_properties(
                  s._info, COLUMNS) for s in self.share_snapshot_instances)

        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(values), list(data))

    def test_share_snapshot_instance_list_detail(self):
        values = (osc_lib_utils.get_dict_properties(
            s._info, COLUMNS_DETAIL) for s in self.share_snapshot_instances)

        arglist = [
            '--detailed'
        ]

        verifylist = [
            ('detailed', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(COLUMNS_DETAIL, columns)
        self.assertEqual(list(values), list(data))

    def test_share_snapshot_instance_list_snapshot_id(self):
        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())

        self.share_snapshots_mock.get.return_value = self.share_snapshot
        self.share_snapshot_instances_mock.list.return_value = (
            [self.share_snapshot_instances[0]])

        values = (osc_lib_utils.get_dict_properties(
            s._info, COLUMNS) for s in [self.share_snapshot_instances[0]])

        arglist = [
            '--snapshot', self.share_snapshot.id
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(values), list(data))


class TestShareSnapshotInstanceShow(TestShareSnapshotInstance):

    def setUp(self):
        super(TestShareSnapshotInstanceShow, self).setUp()

        self.share_snapshot_instance = (
            manila_fakes.FakeShareSnapshotIntances
                        .create_one_snapshot_instance())

        self.share_snapshot_instances_mock.get.return_value = (
            self.share_snapshot_instance)

        self.share_snapshot_instances_el_list = (
            manila_fakes.FakeShareSnapshotInstancesExportLocations
            .create_share_snapshot_instances(count=2)
        )

        self.share_snapshot_instances_el_mock.list.return_value = (
            self.share_snapshot_instances_el_list)

        self.cmd = (osc_share_snapshot_instances
                    .ShowShareSnapshotInstance(self.app, None))

        self.share_snapshot_instance._info['export_locations'] = (
            cliutils.convert_dict_list_to_string(
                self.share_snapshot_instances_el_list))

        self.data = self.share_snapshot_instance._info.values()
        self.columns = self.share_snapshot_instance._info.keys()

    def test_share_snapshot_instance_show_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_snapshot_instance_show(self):
        arglist = [
            self.share_snapshot_instance.id
        ]
        verifylist = [
            ('snapshot_instance', self.share_snapshot_instance.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.share_snapshot_instances_mock.get.assert_called_with(
            self.share_snapshot_instance.id)
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


class TestShareSnapshotInstanceSet(TestShareSnapshotInstance):

    def setUp(self):
        super(TestShareSnapshotInstanceSet, self).setUp()

        self.share_snapshot_instance = (
            manila_fakes.FakeShareSnapshotIntances
                        .create_one_snapshot_instance())

        self.snapshot_instance_status = 'available'

        self.cmd = (osc_share_snapshot_instances
                    .SetShareSnapshotInstance(self.app, None))

    def test_share_snapshot_instance_set_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_snapshot_instance_set_instance_not_found(self):
        arglist = [
            self.share_snapshot_instance.id,
            '--status', self.snapshot_instance_status
        ]
        verifylist = [
            ('snapshot_instance', self.share_snapshot_instance.id),
            ('status', self.snapshot_instance_status)
        ]

        self.share_snapshot_instances_mock.reset_state.side_effect = (
            api_exceptions.NotFound())

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(osc_exceptions.CommandError,
                          self.cmd.take_action, parsed_args)

    def test_share_snapshot_instance_set(self):
        arglist = [
            self.share_snapshot_instance.id,
            '--status', self.snapshot_instance_status
        ]
        verifylist = [
            ('snapshot_instance', self.share_snapshot_instance.id),
            ('status', self.snapshot_instance_status)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.share_snapshot_instances_mock.reset_state.assert_called_with(
            self.share_snapshot_instance.id, self.snapshot_instance_status)



# Copyright 2013 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib import parse

from manilaclient import api_versions
from manilaclient.common import httpclient
from manilaclient.tests.unit import fakes
from manilaclient.tests.unit import utils
from manilaclient.v2 import client


class FakeClient(fakes.FakeClient, client.Client):

    def __init__(self, *args, **kwargs):
        api_version = kwargs.get('version') or api_versions.MAX_VERSION
        client.Client.__init__(self, 'username', 'password',
                               'project_id', 'auth_url',
                               extensions=kwargs.get('extensions'),
                               version=api_version)
        self.client = FakeHTTPClient(**kwargs)


class FakeHTTPClient(httpclient.HTTPClient):

    def __init__(self, **kwargs):
        api_version = kwargs.get('version') or api_versions.MAX_VERSION
        self.username = 'username'
        self.password = 'password'
        self.auth_url = 'auth_url'
        self.callstack = []
        self.base_url = 'localhost'
        self.default_headers = {
            'X-Auth-Token': 'xabc123',
            'X-Openstack-Manila-Api-Version': api_version,
            'Accept': 'application/json',
        }

    def _cs_request(self, url, method, **kwargs):
        return self._cs_request_with_retries(url, method, **kwargs)

    def _cs_request_base_url(self, url, method, **kwargs):
        return self._cs_request_with_retries(url, method, **kwargs)

    def _cs_request_with_retries(self, url, method, **kwargs):
        # Check that certain things are called correctly
        if method in ['GET', 'DELETE']:
            assert 'body' not in kwargs
        elif method == 'PUT':
            assert 'body' in kwargs

        # Call the method
        args = parse.parse_qsl(parse.urlparse(url)[4])
        kwargs.update(args)
        munged_url = url.rsplit('?', 1)[0]
        munged_url = munged_url.strip('/').replace('/', '_').replace('.', '_')
        munged_url = munged_url.replace('-', '_')

        callback = "%s_%s" % (method.lower(), munged_url)

        if not hasattr(self, callback):
            raise AssertionError('Called unknown API method: %s %s, '
                                 'expected fakes method name: %s' %
                                 (method, url, callback))

        # Note the call
        self.callstack.append((method, url, kwargs.get('body', None)))
        status, headers, body = getattr(self, callback)(**kwargs)
        r = utils.TestResponse({
            "status_code": status,
            "text": body,
            "headers": headers,
        })
        return r, body

    #
    # Quotas
    #

    def get_os_quota_sets_test(self, **kw):
        quota_set = {
            'quota_set': {
                'tenant_id': 'test',
                'metadata_items': [],
                'shares': 1,
                'snapshots': 1,
                'gigabytes': 1,
                'snapshot_gigabytes': 1,
                'share_networks': 1,
            }
        }
        return (200, {}, quota_set)

    get_quota_sets_test = get_os_quota_sets_test

    def get_os_quota_sets_test_defaults(self):
        quota_set = {
            'quota_set': {
                'tenant_id': 'test',
                'metadata_items': [],
                'shares': 1,
                'snapshots': 1,
                'gigabytes': 1,
                'snapshot_gigabytes': 1,
                'share_networks': 1,
            }
        }
        return (200, {}, quota_set)

    get_quota_sets_test_defaults = get_os_quota_sets_test_defaults

    def put_os_quota_sets_test(self, body, **kw):
        assert list(body) == ['quota_set']
        fakes.assert_has_keys(body['quota_set'],
                              required=['tenant_id'])
        quota_set = {
            'quota_set': {
                'tenant_id': 'test',
                'metadata_items': [],
                'shares': 2,
                'snapshots': 2,
                'gigabytes': 1,
                'snapshot_gigabytes': 1,
                'share_networks': 1,
            }
        }
        return (200, {}, quota_set)

    put_quota_sets_test = put_os_quota_sets_test

    #
    # Quota Classes
    #

    def get_os_quota_class_sets_test(self, **kw):
        quota_class_set = {
            'quota_class_set': {
                'class_name': 'test',
                'metadata_items': [],
                'shares': 1,
                'snapshots': 1,
                'gigabytes': 1,
                'snapshot_gigabytes': 1,
                'share_networks': 1,
            }
        }
        return (200, {}, quota_class_set)

    get_quota_class_sets_test = get_os_quota_class_sets_test

    def put_os_quota_class_sets_test(self, body, **kw):
        assert list(body) == ['quota_class_set']
        fakes.assert_has_keys(body['quota_class_set'],
                              required=['class_name'])
        quota_class_set = {
            'quota_class_set': {
                'class_name': 'test',
                'metadata_items': [],
                'shares': 2,
                'snapshots': 2,
                'gigabytes': 1,
                'snapshot_gigabytes': 1,
                'share_networks': 1,
            }
        }
        return (200, {}, quota_class_set)

    put_quota_class_sets_test = put_os_quota_class_sets_test

    def delete_os_quota_sets_test(self, **kw):
        return (202, {}, {})

    delete_quota_sets_test = delete_os_quota_sets_test

    #
    # List all extensions
    #

    def get_extensions(self, **kw):
        exts = [
            {
                "alias": "FAKE-1",
                "description": "Fake extension number 1",
                "links": [],
                "name": "Fake1",
                "namespace": ("http://docs.openstack.org/"
                              "/ext/fake1/api/v1.1"),
                "updated": "2011-06-09T00:00:00+00:00"
            },
            {
                "alias": "FAKE-2",
                "description": "Fake extension number 2",
                "links": [],
                "name": "Fake2",
                "namespace": ("http://docs.openstack.org/"
                              "/ext/fake1/api/v1.1"),
                "updated": "2011-06-09T00:00:00+00:00"
            },
        ]
        return (200, {}, {"extensions": exts, })
